package HumanCompare;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Random;

import javax.swing.JOptionPane;

import util.FileUtil;
import domain.AllData;
import domain.CleanType;
import domain.Motion;
import domain.ShotType;

/**
 * Human Judge Utilities
 * 
 * @author David Smith
 *
 */
public class FHJudgeUtil {
	final static String DELIMITER = ",";
	final static String NEWLINE = "\n";
	
	/**
	 * Creates a file with script data that humans can use to compare
	 * @param data An array of AllData, each containing a lined script
	 * @param names the names of the lined script files
	 * @param out the file to output the CSV too
	 * @return True if success
	 * @throws IOException
	 */
	public static boolean CreateFile(AllData[] data, String[] names, File out, int sampleSize) throws IOException {
		int startline;
		int scriptLength;
		liner.LineData line;
		
		int[] order = randomOrder(data.length);
		
		AllData prime = data[order[0]];
		scriptLength = prime.getDataLines().size();
		startline = (int)(Math.random() * (scriptLength - sampleSize - 1));
		
		//create legend file to go with CSV
		String path = out.getAbsolutePath();
		String split = path.split("\\.")[0];
		FileWriter extra = new FileWriter(split + ".txt");
		extra.append("StartLine " + startline + "\r\n");
		for (int i = 0; i < data.length; i++) {
			extra.append("Sample " + (i+1) + ": " + names[order[i]]);
			extra.append("\r\n");
		}
		extra.flush();
		extra.close();
				
		//create the CSV file with the script data
		FileWriter fW = null;
		try {
			fW = new FileWriter(out.getAbsolutePath());
		
		} catch (Exception e) {
			return false;
		}
		for (int i = startline; i < startline + sampleSize; i++) {
			line = prime.getDataLines().get(i);
			fW.append(approved(line.getLine()));
			for (int j = 0; j < order.length; j ++) {
				line = data[order[j]].getDataLines().get(i);
				if (i == startline) {
					fW.append(DELIMITER + "Sample " + (j+1));
				}
				else {
				fW.append(DELIMITER + " ");
				}
				fW.append(DELIMITER);
				fW.append((line.isNewShot()?"cut":"no cut"));
				fW.append(DELIMITER);
				fW.append(ShotType.getValues().get(line.getShotType()));
				fW.append(DELIMITER);
				fW.append(CleanType.getValues().get(line.getCleanType()));
				fW.append(DELIMITER);
				fW.append(Motion.getStrings().get(line.getCleanType()));
			}
			fW.append(NEWLINE);
		}
		fW.flush();
		fW.close();
		
		return true;
	}

	private static int[] randomOrder(int length) {
		int[] order = new int[length];
		for (int i = 0; i < order.length; i ++)
			order[i] = i;
		Random rand = new Random();
		for (int i=0; i<order.length; i++) {
		    int randomPosition = rand.nextInt(order.length);
		    int temp = order[i];
		    order[i] = order[randomPosition];
		    order[randomPosition] = temp;
		}
		return order;
	}
	
	private static String approved(String in) {
		String out = in.replaceAll("\"", "'");
		return "\"" + out + "\"";
	}

	public static String[] getNames(File[] f) {
		String[] n = new String[f.length];
		for (int i = 0; i < f.length; i++)
			n[i] = f[i].getName();
		return n;
	}

	public static File getOutFile() {
		String fileType;
		File out;
		out = FHJudge.getWriteFile();
		if (out == null) 
			return null;
		else {
			fileType = FileUtil.getFileType(out.getName());
			if (!fileType.equals("csv")) {
				JOptionPane.showMessageDialog(null, "Output must be of type .csv");
				return null;
			}
		}
		return out;
	}

	public static AllData[] getAllDatas(File[] f) {
		String fileType;
		AllData[] d = new AllData[f.length];
		for (int i = 0; i < f.length; i ++) {
			if (!f[i].exists()) {
				JOptionPane.showMessageDialog(null, "File " + f[i].getName() + " doesn't exist.");
				return null;
			}
			else {
				fileType = FileUtil.getFileType(f[i].getName());
				if (fileType.equals("zip")) {
					d[i] = FileUtil.getAllDataFromZip(f[i].getAbsolutePath());
				}
				else if (fileType.equals("json")) {
					d[i] = FileUtil.getAllDataFromJson(f[i].getAbsolutePath());
				}
				else {
					JOptionPane.showMessageDialog(null, "Files not of type .zip or .json");
					return null;
				}
			}
		}
		return d;
	}
}
