package comparer;

import java.io.File;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;

import javax.swing.JOptionPane;

import liner.LineData;
import lister.TargetFeature;
import util.FileUtil;
import domain.AllData;

public class CompareUtil {
	
	/**
	 * Compares two files for difference
	 * Displays the results in a CResultsFrame
	 * @param file1
	 * @param file2
	 * @return true if comparison was successful
	 */
	public static boolean compareFiles(File file1, File file2) {
		String fileType1 = FileUtil.getFileType(file1.getName());
		String fileType2 = FileUtil.getFileType(file2.getName());
		AllData data1 = null;
		AllData data2 = null;
		
		if (fileType1.equals("zip")) {
			data1 = FileUtil.getAllDataFromZip(file1.getAbsolutePath());
		}
		else if (fileType1.equals("json")) {
			data1 = FileUtil.getAllDataFromJson(file1.getAbsolutePath());
		}
		else {
			JOptionPane.showMessageDialog(null, "File 1 is not of type .zip or .json");
			return false;
		}
		
		if (fileType2.equals("zip")) {
			data2 = FileUtil.getAllDataFromZip(file2.getAbsolutePath());
		}
		else if (fileType2.equals("json")) {
			data2 = FileUtil.getAllDataFromJson(file2.getAbsolutePath());
		}
		else {
			JOptionPane.showMessageDialog(null, "File 2 is not of type .zip or .json");
			return false;
		}
		
		Comparison comp=getEditDifference(data1, data2);
		comp.setFile1(file1.getName());
		comp.setFile2(file2.getName());
		
		CResultsFrame resultFrame = new CResultsFrame(comp);
		
		return true;
	}
	
	private static Comparison getEditDifference(AllData data1, AllData data2) {
		Comparison comp = new Comparison();
		HashMap<String, Integer> getDiffer1 = getDifference(data1, data2);
		HashMap<String, Integer> getDiffer2 = getDifference(data2, data1);
		for (TargetFeature tF: TargetFeature.values()) {
			comp.setDset1(tF.ordinal(), (double)(getDiffer1.get(tF.name()))); 
			comp.setDset2(tF.ordinal(), (double)(getDiffer2.get(tF.name())));
		}
		comp.calcDifference();
		return comp;
	}
	
	private static HashMap<String, Integer> getDifference(AllData data1, AllData data2) {
		HashMap<String, Integer> difference = new HashMap<String, Integer>();
		for (TargetFeature tF: TargetFeature.values()) {
			difference.put(tF.name(), 0);
		}
		int lineNum = 0;
		for (LineData line: data1.getDataLines()) {
			lineNum = line.getLineNum();
			//this shouldn't happen if they're the same script
			if (line.getLineNum() >= data2.getDataLines().size())
				continue;
			
			if (line.isNewShot() && !data2.getDataLines().get(lineNum).isNewShot()) {
				int up = toCutLineCount(lineNum, -1, data2);
				int down = toCutLineCount(lineNum, 1, data2);
				int min = Math.min(up, down);
				System.out.println(min);
				difference.put(TargetFeature.Cut.name(),
						difference.get(TargetFeature.Cut.name()) + min);
			}
			
			if (line.getShotType() != data2.getDataLines().get(lineNum).getShotType()) {
				int up = toShotLineCount(line.getShotType(), lineNum, -1, data2);
				int down = toShotLineCount(line.getShotType(), lineNum, 1, data2);
				int min = Math.min(up, down);
				difference.put(TargetFeature.ShotType.name(),
						difference.get(TargetFeature.ShotType.name()) + min);
			}
			
			if (line.getCleanType() != data2.getDataLines().get(lineNum).getCleanType()) {
				int up = toCleanLineCount(line.getCleanType(), lineNum, -1, data2);
				int down = toCleanLineCount(line.getCleanType(), lineNum, 1, data2);
				int min = Math.min(up, down);
				difference.put(TargetFeature.CleanType.name(),
						difference.get(TargetFeature.CleanType.name()) + min);
			}
			
			if (line.getMotion() != data2.getDataLines().get(lineNum).getMotion()) {
				int up = toMotionLineCount(line.getMotion(), lineNum, -1, data2);
				int down = toMotionLineCount(line.getMotion(), lineNum, 1, data2);
				int min = Math.min(up, down);
				difference.put(TargetFeature.Motion.name(),
						difference.get(TargetFeature.Motion.name()) + min);
			}
		}
		return difference;
	}
	
	private static int toCutLineCount(int startLine, int iteration, AllData data2) {
		int count = 0;
		int cur = startLine;
		while (!data2.getDataLines().get(cur).isNewShot()) {
			count++;
			cur += iteration;
			if (cur < 0 || cur >= data2.getDataLines().size())
				return count;
		}
		return count;
	}
	
	private static int toShotLineCount(int type, int startLine, int iteration, AllData data2) {
		int count = 0;
		int cur = startLine;
		while (data2.getDataLines().get(cur).getShotType() != type) {
			count++;
			cur += iteration;
			if (cur < 0 || cur >= data2.getDataLines().size())
				return count;
		}
		return count;
	}
	
	private static int toCleanLineCount(int type, int startLine, int iteration, AllData data2) {
		int count = 0;
		int cur = startLine;
		while (data2.getDataLines().get(cur).getCleanType() != type) {
			count++;
			cur += iteration;
			if (cur < 0 || cur >= data2.getDataLines().size())
				return count;
		}
		return count;
	}
	
	private static int toMotionLineCount(int type, int startLine, int iteration, AllData data2) {
		int count = 0;
		int cur = startLine;
		while (data2.getDataLines().get(cur).getMotion() != type) {
			count++;
			cur += iteration;
			if (cur < 0 || cur >= data2.getDataLines().size())
				return count;
		}
		return count;
	}
	
}
