package liner;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import domain.BlankLine;
import domain.Dialogue;
import domain.Motion;
import domain.PreScript;
import domain.SceneObject;
import domain.Script;
import domain.ScriptObject;
import domain.ScriptObjectType;
import domain.ShotType;
import domain.CleanType;
import domain.VisibleObject;

public class JLine extends JPanel {
	
	private Script script;
	private FormValues values;
	
	private JLabel lineNum = new JLabel();
	private JTextField timeCode = new JTextField();
	private JLabel sceneNum = new JLabel();
	private JPanel selections = new JPanel();
	private JCheckBox newShot = new JCheckBox();
	private JComboBox shotType = new JComboBox(ShotType.values());
	private JComboBox cleanType = new JComboBox(CleanType.values());
	private JComboBox motion = new JComboBox(Motion.values());
	private JLabel scriptLine = new JLabel();
	private LineData data = new LineData();
	
	private JPanel objects = new JPanel();
	private List<LineObject> lineObjects = new ArrayList<LineObject>();
	
	private List<LineData> dataLines;
	private ScriptForm form;

	private boolean active;
	private int old;
	
	/**
	 * Sets this line as current if set is true, not if false
	 * @param set whether to set this line as current
	 */
	public void setCur(boolean set) {
		if (set) {
			lineNum.setOpaque(true);
			lineNum.setBackground(values.getCurLineColor());
		}
		else {
			lineNum.setOpaque(false);
			lineNum.setBackground(Color.WHITE);
		}
	}
	
	/**
	 * Set the color of the current line
	 */
	public void setColor() {
		if (data.getSceneNum()!=null) {
			this.setOpaque(true);
			this.setBackground(values.getNewSceneColor());
		}
		else if (data.isNewShot()) {
			this.setOpaque(true);
			this.setBackground(values.getCutLineColor());
		}
		else if (!isSame()) {
			this.setOpaque(true);
			this.setBackground(values.getChangeLineColor());
		}
		else {
			this.setOpaque(false);
			this.setBackground(Color.WHITE);
		}
	}
	
	/**
	 * Updates the graphical interpretation of the line based on the data
	 * @param data the data of the current line
	 */
	public void update(LineData data) {
		this.data = data;
		lineNum.setText("Line " + data.getLineNum());
		if (data.getSceneNum() != null)
			sceneNum.setText(" " + data.getSceneNum());
		else
			sceneNum.setText("");
		if (data.getTimeCode() != null)
			timeCode.setText(data.getTimeCode());
		else
			timeCode.setText("");
		newShot.setToolTipText("New shot");
		newShot.setSelected(data.isNewShot());
		newShot.setEnabled(true);
		if (shotType.getSelectedIndex()!=data.getShotType())
			shotType.setSelectedIndex(data.getShotType());
		shotType.setEnabled(true);
		if (cleanType.getSelectedIndex()!=data.getCleanType())
			cleanType.setSelectedIndex(data.getCleanType());
		cleanType.setEnabled(true);
		if (motion.getSelectedIndex()!=data.getMotion())
			motion.setSelectedIndex(data.getMotion());
		motion.setEnabled(true);
		scriptLine.setText(data.getLine());
		SceneObject sO = script.getSceneObjects().get(script.getLines()[data.getLineNum()]);
		scriptLine.setToolTipText(generateHint(sO));
		form.requestFocus();
		objects.removeAll();
		lineObjects.clear();
		for (VisibleObject obj: data.getVisibleObjects()) {
			if (printable(obj)) {
				LineObject object = new LineObject(obj, dataLines, form, script, this);
				object.getNameLabel().setOpaque(true);
				object.getVisibleBox().setSelected(obj.isVisible());
					object.switchToBox();
				lineObjects.add(object);
				objects.add(object);
			}
		}
		setColor();
		this.revalidate();
		this.setVisible(true);
	}
	
	private String generateHint(SceneObject sO) {
		String type = sO.getType();
		if (type.equals(BlankLine.TYPE) || type.equals(PreScript.TYPE)) {
			return type;
		}
		else if (type.equals(Dialogue.TYPE)) {
			return type + " - Scene " + sO.getScene() + " - Character " + 
					script.getObjects().get(((Dialogue)sO).getCharacter()).getName();
		}
		else
			return type + " - Scene " + sO.getScene();
	}
	
	/**
	 * Updates the header
	 * @param data the data to use to update the header
	 */
	public void updateHeader(LineData data) {
		lineNum.setText("Scene");
		sceneNum.setText(" " + data.getSceneNum());
		sceneNum.setOpaque(false);
		timeCode.setText("--:--:--");
		timeCode.setEnabled(false);
		newShot.setVisible(false);
		shotType.setVisible(false);
		cleanType.setVisible(false);
		motion.setVisible(false);
		selections.setOpaque(false);
		scriptLine.setText(data.getLine());
		objects.removeAll();
		lineObjects.clear();
		for (VisibleObject obj: data.getVisibleObjects()) {
			if (printable(obj)) {
				LineObject object = new LineObject(obj, dataLines, form, script, this);
				object.getVisibleBox().setSelected(obj.isVisible());
				lineObjects.add(object);
				objects.add(object);
			}
		}
		this.revalidate();
		this.setVisible(true);
	}
	
	private boolean printable(VisibleObject obj) {
		ScriptObject o = script.getObject(obj.getObjectIndex());
		return o.getType().isPrintable();
	}
	
	/**
	 * Sets the opener line
	 * @param line the string to set the line to
	 */
	public void setOpener(String line) {
		updateBlank();
		scriptLine.setText(line);
	}
	
	/**
	 * Updates the line as blank
	 */
	public void updateBlank() {
		this.data = null;
		lineNum.setText("");
		sceneNum.setText("");
		newShot.setSelected(false);
		newShot.setEnabled(false);
		shotType.setSelectedItem(0);
		shotType.setEnabled(false);
		cleanType.setSelectedItem(0);
		cleanType.setEnabled(false);
		motion.setSelectedItem(0);
		motion.setEnabled(false);
		scriptLine.setText("");
		objects.removeAll();
		lineObjects.clear();
		this.setVisible(false);
	}
	
	public JLine(ScriptForm inForm, List<LineData> inDataLines, int lineLength, Script inScript, FormValues inValues) { //, final ScriptForm sForm) {
		script = inScript;
		values = inValues;
		form = inForm;
		dataLines = inDataLines;
		
		String placeholderLen = "1000";
		setLayout(new FlowLayout(FlowLayout.LEFT));
		
		lineNum.setText("Line " + placeholderLen);
		Dimension d = lineNum.getPreferredSize();
	    lineNum.setPreferredSize(new Dimension(d.width,d.height));
	    lineNum.setBackground(Color.WHITE);
	    lineNum.setOpaque(false);
	    
	    timeCode.setText("00:00:00");
	    d = timeCode.getPreferredSize();
	    timeCode.setPreferredSize(new Dimension(d.width,d.height));
	    
	    sceneNum.setText(placeholderLen);
		d = sceneNum.getPreferredSize();
	    sceneNum.setPreferredSize(new Dimension(d.width,d.height));
	    
	    StringBuilder lineHolder = new StringBuilder();
	    for (int i = 0; i<lineLength; i++)
	    	lineHolder.append("  ");
	    scriptLine.setText(lineHolder.toString());
		Dimension d2 = scriptLine.getPreferredSize();
	    scriptLine.setPreferredSize(new Dimension(d2.width,d2.height));
	    
	    TipRenderer renderer = new TipRenderer();
	    shotType.setRenderer(renderer);
	    renderer.setTips(ShotType.getStrings());
	    
	    TipRenderer renderer2 = new TipRenderer();
	    cleanType.setRenderer(renderer2);
	    renderer2.setTips(CleanType.getStrings());
	    
	    timeCode.setToolTipText("Optional - Put timecode here");
	    shotType.setToolTipText("Choose the shot type");
	    cleanType.setToolTipText("Choose how the characters/objects are in the frame");
	    motion.setToolTipText("Choose the camera motion");
	    
	    timeCode.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						String timeC;
						if (timeCode.getText().isEmpty())
							timeC = null;
						else
							timeC = timeCode.getText();
						data.setTimeCode(timeC);
						form.requestFocus();
					}
				});
	    
		newShot.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						data.setNewShot(newShot.isSelected());
						setColor();
						if (form.isAltPressed() && newShot.isSelected())
							copyPreviousCut(1);
						else if (form.isCtrlPressed() && newShot.isSelected())
							copyPreviousCut(2);
						form.requestFocus();
					}
				}
				);
		shotType.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						if (active) {
							old = data.getShotType();
							data.setShotType(shotType.getSelectedIndex());
							int index = data.getLineNum() + 1;
							while (index < dataLines.size()
									&& !dataLines.get(index).isNewShot()
									&& dataLines.get(index).getShotType()==old) {
								dataLines.get(index).setShotType(shotType.getSelectedIndex());
								index++;
							}
							form.updateLines();
						}
						form.requestFocus();
						
					}
				}
				);
		cleanType.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						if (active) {
							old = data.getCleanType();
							data.setCleanType(cleanType.getSelectedIndex());
							int index = data.getLineNum() + 1;
							while (index < dataLines.size()
									&& !dataLines.get(index).isNewShot()
									&& dataLines.get(index).getCleanType()==old) {
								dataLines.get(index).setCleanType(cleanType.getSelectedIndex());
								index++;
							}
							form.updateLines();
						}
						form.requestFocus();
					}
				}
				);
		motion.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						if (active) {
							old = data.getMotion();
							data.setMotion(motion.getSelectedIndex());
							int index = data.getLineNum() + 1;
							while (index < dataLines.size()
									&& !dataLines.get(index).isNewShot() 
									&& dataLines.get(index).getMotion()==old) {
								dataLines.get(index).setMotion(motion.getSelectedIndex());
								index++;
							}
							form.updateLines();
						}
						form.requestFocus();
					}
				}
				);
		
		add(lineNum);
		add(sceneNum);
		add(timeCode);
		selections.add(newShot);	
		selections.add(shotType);
		selections.add(cleanType);
		selections.add(motion);
		add(selections);
		d = selections.getPreferredSize();
	    selections.setPreferredSize(new Dimension(d.width,d.height));
		
		
		add(scriptLine);
		add(objects);
	}

	
	public JTextField getTimeCode() {
		return timeCode;
	}

	public void setTimeCode(JTextField timeCode) {
		this.timeCode = timeCode;
	}

	public JCheckBox getNewShot() {
		return newShot;
	}

	public void setNewShot(JCheckBox newShot) {
		this.newShot = newShot;
	}

	public JComboBox getShotType() {
		return shotType;
	}

	public void setShotType(JComboBox shotType) {
		this.shotType = shotType;
	}

	public JComboBox getCleanType() {
		return cleanType;
	}

	public void setCleanType(JComboBox cleanType) {
		this.cleanType = cleanType;
	}

	public JComboBox getMotion() {
		return motion;
	}

	public void setMotion(JComboBox motion) {
		this.motion = motion;
	}

	public boolean isActive() {
		return active;
	}

	/**
	 * Sets the line as the active line or not active
	 * @param active whether to set the line active or not active
	 */
	public void setActive(boolean active) {
		this.active = active;
		for (LineObject lineObject: lineObjects )
			lineObject.setActive(active);
	}
	
	/**
	 * Toggles whether the line is a new shot
	 */
	public void toggleNewShot() {
		newShot.setSelected(!newShot.isSelected());
		data.setNewShot(newShot.isSelected());
	}
	
	/**
	 * Advances the shot, either up or down
	 * @param down advance down if true or up if false
	 */
	public void advanceShotType(boolean down) {
		int type = shotType.getSelectedIndex();
		int max = shotType.getItemCount();
		if (down) {
			if (type >= max - 1)
				type = 0;
			else
				type++;
		}
		else {
			if (type <= 0)
				type = max - 1;
			else
				type--;
		}
		shotType.setSelectedIndex(type);
		data.setShotType(type);
	}
	
	/**
	 * Advances the clean type, either up or down
	 * @param down advance down if true or up if false
	 */
	public void advanceCleanType(boolean down) {
		int type = cleanType.getSelectedIndex();
		int max = cleanType.getItemCount();
		if (down) {
			if (type >= max - 1)
				type = 0;
			else
				type++;
		}
		else {
			if (type <= 0)
				type = max - 1;
			else
				type--;
		}
		cleanType.setSelectedIndex(type);
		data.setCleanType(type);
	}
	
	/**
	 * Advances the motion, either up or down
	 * @param down advance down if true or up if false
	 */
	public void advanceMotion(boolean down) {
		int type = motion.getSelectedIndex();
		int max = motion.getItemCount();
		if (down) {
			if (type >= max - 1)
				type = 0;
			else
				type++;
		}
		else {
			if (type <= 0)
				type = max - 1;
			else
				type--;
		}
		motion.setSelectedIndex(type);
		data.setMotion(type);
	}
	
	/**
	 * Toggles the character on or off
	 * @param character the character to toggle
	 */
	public void toggleCharacter(int character) {
		if (lineObjects.size() > character)
			lineObjects.get(character).toggleVisible();
	}
	
	/**
	 * toggles all the same
	 * @param index the index to toggle
	 */
	public void toggleAllSame(int index) {
		boolean old = true;
		for (LineObject obj: lineObjects)
			if (obj.getIndex() == index)
				old = obj.isObjectVisible();
//		boolean old = true;
		for (int i = 0; i < lineObjects.size(); i++) {
			lineObjects.get(i).setVisible(!old);
		}
	}
	
	/**
	 * toggle all different from index
	 * @param index the index to set different
	 */
	public void toggleAllDifferent(int index) {
		boolean old = true;
		for (LineObject obj: lineObjects)
			if (obj.getIndex() == index)
				old = obj.isObjectVisible();
//		boolean old = true;
		for (int i = 0; i < lineObjects.size(); i++) {
			if (lineObjects.get(i).getIndex() == index)
				lineObjects.get(i).setVisible(!old);
			else
				lineObjects.get(i).setVisible(old);
		}
	}
	
	public List<LineObject> getLineObjects() {
		return lineObjects;
	}
	
	public boolean isNewShot() {
		return newShot.isSelected();
	}
	
	/**
	 * copies this line to another line
	 * @param other the other line to copy to
	 */
	public void copyJLine(JLine other) {
		
		newShot.setSelected(other.getNewShot().isSelected());
		shotType.setSelectedIndex(other.getShotType().getSelectedIndex());
		cleanType.setSelectedIndex(other.getCleanType().getSelectedIndex());
		motion.setSelectedIndex(other.getMotion().getSelectedIndex());
		int size = lineObjects.size();
		for (int i = 0; i < size; i++) {
			if (other.getLineObjects().size() > i)
				lineObjects.get(i).setVisible(other.getLineObjects().get(i).getVisibleBox().isSelected());
		}
	}
	
	/**
	 * copy the previous cut
	 * @param stop line to stop
	 */
	public void copyPreviousCut(int stop) {
		int backCount = 0;
		int cur = data.getLineNum();
		while (backCount < stop && cur > 1) {
			cur --;
			if(dataLines.get(cur).isNewShot())
				backCount++;
		}
		if (backCount != stop)
			return;
		LineData other = dataLines.get(cur - 1);
		shotType.setSelectedIndex(other.getShotType());
		cleanType.setSelectedIndex(other.getCleanType());
		motion.setSelectedIndex(other.getMotion());
		int size = lineObjects.size();
		List<VisibleObject> objs = new ArrayList<VisibleObject>();
		for (VisibleObject obj: other.getVisibleObjects())
			if (printable(obj))
				objs.add(obj);
		for (int i = 0; i < size; i++) {
			if (objs.size() > i)
				lineObjects.get(i).setVisible(objs.get(i).isVisible());
		}
	}
	
	private boolean isSame() {
		if (data.getLineNum() > 0 && !data.isNewShot()) {
			LineData prev = dataLines.get(data.getLineNum()-1);
			if (data.getShotType() != prev.getShotType())
				return false;
			if (data.getCleanType() != prev.getCleanType())
				return false;
			if (data.getMotion() != prev.getMotion())
				return false;
			int size = data.getVisibleObjects().size();
			for (int i = 0; i < size; i++)
				if (prev.getVisibleObjects().size() > i)
					if (data.getVisibleObjects().get(i).isVisible() !=
					prev.getVisibleObjects().get(i).isVisible())
						return false;
		}
		return true;
	}
	
}
