package liner;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import domain.Script;
import domain.ScriptObject;
import domain.VisibleObject;

public class LineObject extends JPanel {
	
	Script script;
	ScriptObject object;
	
	JLabel name = new JLabel();
	JCheckBox visible = new JCheckBox();
	Dimension d;
	
	private VisibleObject data;
	private List<LineData> dataLines;
	private ScriptForm form;
	private JLine jLine;
	int objectIndex;
//	int index;
	
	boolean active = true;
	
	public LineObject(VisibleObject inData, List<LineData> inDataLines, ScriptForm inForm, final Script script, JLine inJLine) {
		this.script = script;
		data = inData;
		dataLines = inDataLines;
		form = inForm;
		jLine = inJLine;
		
		int width = 6;
		StringBuilder filler = new StringBuilder();
		for (int i=0; i<width; i++)
			filler.append("X");
		name.setText(filler.toString());
		
		add(name);
		d = getPreferredSize();
	    name.setPreferredSize(new Dimension(name.getPreferredSize().width,name.getPreferredSize().height));
	    objectIndex = data.getObjectIndex();
//	    index = data.getIndex();
	    object = script.getObject(objectIndex);
	    name.setText(object.getName());
	    if (object.getName().length() < width)
	    	name.setHorizontalAlignment(JLabel.CENTER);
	    else
	    	name.setHorizontalAlignment(JLabel.LEFT);
	    name.setToolTipText(object.getIndex() + " " + object.getName());
	    
	    visible.setText(object.getName());
		visible.setSelected(false);
		if (object.getColor().isDark())
			visible.setForeground(Color.WHITE);
		visible.setToolTipText(object.getIndex() + " " + object.getName());
		visible.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						if (form.isAltPressed()) {
							jLine.toggleAllDifferent(data.getIndex());
						}
						else if (form.isCtrlPressed()) {
							jLine.toggleAllSame(data.getIndex());
						}
						else 
							toggleVisible();
						form.requestFocus();
					}
				}
				);
//		setLayout(new FlowLayout());
//		add(visible);
	}
	
	public void switchToBox() {
		remove(name);
		setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.Y_AXIS));
		add(visible);
		setPreferredSize(new Dimension(d.width,d.height));
		this.setOpaque(true);
		this.setBackground(object.getColor().asColor());
		visible.setOpaque(false);
	}
	
	public JLabel getNameLabel() {
		return name;
	}
	public void setNameLabel(JLabel nameLabel) {
		this.name = nameLabel;
	}
	public JCheckBox getVisibleBox() {
		return visible;
	}
	public void setVisibleBox(JCheckBox isVisible) {
		this.visible = isVisible;
	}

	public boolean isActive() {
		return active;
	}

	public void setActive(boolean active) {
		this.active = active;
	}
	
	public void toggleVisible() {
		if (active) {
			int lineNum = data.getLineNum();
			int index = data.getIndex();
//			System.out.println("set visible index: " + index);
			boolean old = dataLines.get(lineNum).getVisibleObjects().get(index).isVisible();
			data.setVisible(!old);
			
			//set the previous one if it's hidden
//			if (dataLines.get(lineNum-1).getSceneNum()!=null)
//				dataLines.get(lineNum-1).getVisibleObjects().get(index).setVisible(!old);
			
			lineNum++;
			while (lineNum < script.getScriptLength()
					&& !dataLines.get(lineNum).isNewShot()
					&& dataLines.get(lineNum).getVisibleObjects().get(index).isVisible()==old) {
				dataLines.get(lineNum).getVisibleObjects().get(index).setVisible(!old);
				lineNum++;
			}
			form.updateLines();
			form.requestFocus();
		}
	}
	
	public void setVisible(boolean isVisible) {
		if (active) {
			int lineNum = data.getLineNum();
			int index = data.getIndex();
//			System.out.println("set visible index: " + index);
			boolean old = dataLines.get(lineNum).getVisibleObjects().get(index).isVisible();
			data.setVisible(isVisible);
			lineNum++;
			while (lineNum < script.getScriptLength()
					&& !dataLines.get(lineNum).isNewShot()
					&& dataLines.get(lineNum).getVisibleObjects().get(index).isVisible()==old) {
				dataLines.get(lineNum).getVisibleObjects().get(index).setVisible(isVisible);
				lineNum++;
			}
			form.updateLines();
			form.requestFocus();
		}
	}
	
	public boolean isObjectVisible() {
		return dataLines.get(data.getLineNum()).getVisibleObjects().get(data.getIndex()).isVisible();
	}
	
	public int getIndex() {
		return data.getIndex();
	}
	
}
