package liner;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import parser.Parser;
import util.FileUtil;
import util.Util;
import domain.AllData;
import domain.Scene;
import domain.SceneObject;
import domain.Script;
import domain.ScriptObject;
import domain.VisibleObject;

public class LinerUtil {

	public static AllData createDataLines(List<String> lines, Script script) {
		int max = 0;
		List<LineData> dataLines = new ArrayList<LineData>();
		LineData lineData;
		int lineNum=0;
		int sceneNum = 0;
		for (String line: lines) {
			if (line.length() > max)
				max = line.length();
			lineData = new LineData(line, lineNum);
			if (Util.isScene(line)) {
				lineData.setLine(Util.removeFirstAndLastNum(line));
				sceneNum++;
				lineData.setSceneNum(sceneNum);
				lineData.setNewShot(true);
			}
			if (sceneNum > 0 && sceneNum <= script.getScenes().size()) {
				int index = 0;
				for(Integer obj: script.getScenes().get(sceneNum-1).getObjects()) {
					lineData.getVisibleObjects().add(new VisibleObject(obj, false, index, lineNum));
					index++;
				}
			}
			lineNum++;
			dataLines.add(lineData);
		}
		AllData returnData = new AllData(dataLines, script, max);
		return returnData;
	}
	
	public static void updateDataLines(AllData data) {
		int lineNum=0;
		int sceneNum = 0;
		for (LineData line: data.getDataLines()) {
			if (line.getSceneNum()!=null) {
				sceneNum = line.getSceneNum();
			}
			if (sceneNum > 0 ) {
				for(Integer obj: data.getScript().getScenes().get(sceneNum-1).getObjects()) {
					line.addVisibleObject(obj);
				}
			}
			lineNum++;
		}
	}
	
	public static AllData restoreAllData(AllData data) {
		int index;
		int lineNum = 0;
		int sceneNum = 0;
		List<LineData> dataLines = data.getDataLines();
		for (LineData line: dataLines) {
			if (Util.isScene(line.getLine())) {
				sceneNum++;
				line.setSceneNum(sceneNum);
			}
			line.setLineNum(lineNum);
			index = 0;
			for (VisibleObject obj:line.getVisibleObjects()) {
				obj.setIndex(index);
				obj.setLineNum(lineNum);
				index++;
			}
			lineNum++;
		}
		index = 0;
		for (ScriptObject obj: data.getScript().getObjects()) {
			obj.setIndex(index);
			index++;
		}
		index = 0;
		for (SceneObject obj: data.getScript().getSceneObjects()) {
			obj.setIndex(index);
			index++;
		}
		for (Scene scene: data.getScript().getScenes()) {
			for (Integer s: scene.getSceneObjects()) {
				data.getScript().getSceneObjects().get(s).setScene(scene.getId());
			}
		}
		return data;
	}

	/**
	 * Adds an object to the script not originally found by the parser.
	 * @param name The name of the object being added
	 * @param data all the data
	 * @return The new script object, or null, if it's not in the script
	 */
	public static ScriptObject addObjectToScript(String name, AllData data) {
		String newName = name.toUpperCase();
		Integer sceneNum = 0;
		ScriptObject obj = null;
		for (LineData d: data.getDataLines()) {
			String line = d.getLine();
			if (Util.isScene(line))
				sceneNum++;
			else if (sceneNum >=1 
					&& line.toUpperCase().matches(".*\\b" + newName + "\\b.*")) 
			{
				obj = data.getScript().addObject(newName, sceneNum);
				data.getScript().getScenes().get(sceneNum-1).addScriptObject(obj.getIndex());
			}
		}
		return obj;
	}

	/**
	 * Goes through every line of the script and checks if an existing object was missed
	 * in an earlier scene than it was introduced, and adds it to that scene if it was.
	 * @param data
	 */
	public static boolean recheckScriptObjects(AllData data) {
		boolean change = false;
		Integer sceneNum = 0;
		for (LineData d: data.getDataLines()) {
			String line = d.getLine();
			if (Util.isScene(line)) 
				sceneNum++;
			else if (sceneNum >= 1 && sceneNum <= data.getScript().getScenes().size())
				if(data.getScript().getSceneObjects().get(data.getScript().getLines()[d.getLineNum()]).getType()
						.equals("action block")) {
					for(ScriptObject obj: data.getScript().getObjects()) {
						if (line.toUpperCase().matches(".*\\b" + obj.getName() + "\\b.*")) {
							obj.addScene(sceneNum);
							data.getScript().getScenes().get(sceneNum-1).addScriptObject(obj.getIndex());
							change = true;
						}
					}
				}
		}
		return change;
	}

	public static void openLoadScriptForms(File file, boolean openedElsewhere) {
		String fileType = FileUtil.getFileType(file.getName());
		AllData allData;
		if (fileType.equals("json")) 
			allData = FileUtil.getAllDataFromJson(file.getAbsolutePath());
		else 
			allData = FileUtil.getAllDataFromZip(file.getAbsolutePath());
		final ScriptForm sForm = new ScriptForm(allData, Liner.linesPerPage, openedElsewhere);
		final ObjectForm oForm = new ObjectForm(allData, sForm, openedElsewhere);
		sForm.setObjectForm(oForm);
		sForm.setVisible(true);
		oForm.setVisible(true);
	}
	
	public static void openNewScriptForms(File file, boolean openedElsewhere) {
		String name = FileUtil.getName(file.getName());
		AllData allData;

		ArrayList<String> text = FileUtil.readFile(file.getAbsolutePath());

		Script script = Parser.textToScript(text, name);

		allData = LinerUtil.createDataLines(text, script);

		if(LinerUtil.recheckScriptObjects(allData))
			LinerUtil.updateDataLines(allData);

		final ScriptForm sForm = new ScriptForm(allData, Liner.linesPerPage, openedElsewhere);
		final ObjectForm oForm = new ObjectForm(allData, sForm, openedElsewhere);
		sForm.setObjectForm(oForm);
		
		final MovieForm mForm = new MovieForm(script, sForm, oForm, openedElsewhere);
	}
	
}
