package liner;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import liner.ObjectForm.ObjectLine;
import domain.AllData;
import domain.Scene;
import domain.SceneObject;
import domain.Script;
import domain.ScriptObject;
import domain.ScriptObjectType;
import domain.ShotType;

public class ObjectForm extends JFrame
{
	
	public class ObjectLine extends JPanel {
		ScriptObject object;
		JLabel colorBox;
		JLabel l;
		JComboBox objectType;
		JComboBox repeatOf;
		JLabel scenes;
		JTextField newScene;
		JButton sceneButton;
		Integer index;
		Script script;
		
		public ObjectLine(ScriptObject inObject, Script inScript, int inIndex) {
			setTitle("Script Object List");
			index = inIndex;
			object = inObject;
			setLayout(new FlowLayout(FlowLayout.LEFT));
			script = inScript;
			
			colorBox = new JLabel("XX");
			Dimension d = colorBox.getPreferredSize();
			colorBox.setPreferredSize(new Dimension(d.width, d.height));
			colorBox.setText("");
			colorBox.setOpaque(true);
			colorBox.setBackground(object.getColor().asColor());
			
			l = new JLabel(object.getIndex() + " " + object.getName());
			objectType = new JComboBox(ScriptObjectType.values());
			objectType.setSelectedItem(object.getType());
			TipRenderer renderer = new TipRenderer();
		    objectType.setRenderer(renderer);
		    renderer.setTips(ScriptObjectType.getStrings());
			objectType.addActionListener(
					new ActionListener(){
						@Override
						public void actionPerformed(
								ActionEvent e){
							if (objectType.getSelectedItem().equals(ScriptObjectType.Repeat)
									&& !repeatOf.isVisible()) {
								repeatOf.setVisible(true);
								repeatOf.setSelectedIndex(index);
								update();
							}
							else if (!objectType.getSelectedItem().equals(ScriptObjectType.Repeat)
									&& repeatOf.isVisible()) {
								repeatOf.setVisible(false);
								update();
							}
								
						}
					});
			String[] repeatOptions = new String[script.getObjects().size()];
			int i = 0;
			for (ScriptObject obj: script.getObjects()) {
				repeatOptions[i] = obj.getName();
				i++;
			}
			repeatOf = new JComboBox(repeatOptions);
			repeatOf.setPrototypeDisplayValue("XXXXXXXXXX");
			renderer = new TipRenderer();
		    repeatOf.setRenderer(renderer);
		    renderer.setTips(new ArrayList<String>(Arrays.asList(repeatOptions)));
			
			if (object.getType().equals(ScriptObjectType.Repeat)) {
				repeatOf.setVisible(true);
				repeatOf.setSelectedIndex(object.getRepeatOf());
			}
			else
				repeatOf.setVisible(false);
			StringBuilder scenesIn = new StringBuilder("");
			for (Integer scene: object.getScenes())
				scenesIn.append(" " + scene);
			scenes = new JLabel(scenesIn.toString());
			newScene = new JTextField("100");
			d = newScene.getPreferredSize();
			newScene.setPreferredSize(new Dimension(d.width, d.height));
			newScene.setText("");
			sceneButton = new JButton("add/remove");
			sceneButton.addActionListener(
					new ActionListener(){
						@Override
						public void actionPerformed(
								ActionEvent e){
							String content = newScene.getText();
							newScene.setText("");
							Integer scene;
							try {
								scene = Integer.parseInt(content);
							} catch (Exception e1) {
								JOptionPane.showMessageDialog(null, "Invalid scene number");
								return;
							}
							if (scene < 1 || scene > script.getScenes().size())
								JOptionPane.showMessageDialog(null, "Scene must be from 1 to " + script.getScenes().size());
							else {
								boolean remove;
								List<Integer> scenes = script.getObject(index).getScenes();
								int i = 0;
								while (i < scenes.size() && scenes.get(i) < scene)
									i++;
								if (i < scenes.size() && scenes.get(i).equals(scene)) {
									remove = true;
									scenes.remove(i);
									script.getScenes().get(scene-1).removeScriptObject(index);
								}
								else {
									remove = false;
									scenes.add(i, scene);
									script.getScenes().get(scene-1).addScriptObject(index);
								}
								int j = 0;
								while (j < allData.getDataLines().size()
										&& (allData.getDataLines().get(j).getSceneNum() == null
										|| !(allData.getDataLines().get(j).getSceneNum().equals(scene)))
										) {
									j++;
								}
//								System.out.println("index " + j);
//								System.out.println("sceneNum " + allData.getDataLines().get(j).getSceneNum());
								while (j < allData.getDataLines().size() &&
										(allData.getDataLines().get(j).getSceneNum() == null
										|| allData.getDataLines().get(j).getSceneNum().equals(scene))) {
									if (remove)
										allData.getDataLines().get(j).removeVisibleObject(index);
									else
										allData.getDataLines().get(j).addVisibleObject(index);
									j++;
								}
								update();
								updateScript(script);
								form.update(PaneUpdate.REDRAW);
							}
						}
					});
			update();
		}
			
		public void update() {
			removeAll();
			add(colorBox);
			add(l);
			add(objectType);
			add(repeatOf);
			add(scenes);
			add(newScene);
			add(sceneButton);
		}

		public JLabel getL() {
			return l;
		}

		public void setL(JLabel l) {
			this.l = l;
		}

		public JComboBox getObjectType() {
			return objectType;
		}

		public void setObjectType(JComboBox objectType) {
			this.objectType = objectType;
		}

		
		public JComboBox getRepeatOf() {
			return repeatOf;
		}

		public void setRepeatOf(JComboBox repeatOf) {
			this.repeatOf = repeatOf;
		}

		public ScriptObject getObject() {
			return object;
		}

		public void setObject(ScriptObject object) {
			this.object = object;
		}
		
	}
	
	AllData allData;
	Script script;
	public final ScriptForm form;
	
	JPanel listPane = new JPanel();
	JScrollPane listScroller;
	JScrollBar verticalScrollBar;
	
	JPanel bottomPane;
	JButton update = new JButton("update");
	
	JLabel addLabel = new JLabel(" add an Object:");
	JTextField addText = new JTextField("NewObject");
	JButton addButton = new JButton("add");
	
	
	public List<ObjectLine> objectLines = new ArrayList<ObjectLine>();

	public ObjectForm(final AllData all, ScriptForm next) {
		this(all, next, false);
	}
	
	public ObjectForm(final AllData all, ScriptForm next, boolean openedElsewhere) {
		allData = all;
		script = all.getScript();
		form = next;
		listPane.setLayout(new BoxLayout(listPane, BoxLayout.Y_AXIS));
		
		update();
		listScroller = new JScrollPane(listPane);
		listScroller.setAlignmentX(LEFT_ALIGNMENT);
		verticalScrollBar = listScroller.getVerticalScrollBar();
		add(listScroller, BorderLayout.CENTER);
		
		bottomPane = new JPanel();
		update.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						updateScript(script);
						form.update(PaneUpdate.REDRAW);
					}
					}
				);
		bottomPane.add(update);
		bottomPane.add(addLabel);
		bottomPane.add(addText);
		addText.setPreferredSize(new Dimension(addText.getPreferredSize().width, 
				addText.getPreferredSize().height));
		addText.setText("");
		bottomPane.add(addButton);
		addButton.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						int lastIndex = allData.getScript().getObjects().size();
						ScriptObject obj = LinerUtil.addObjectToScript(addText.getText(), allData);
						if(obj == null) {
							JOptionPane.showMessageDialog(null, "Object \""+ addText.getText() 
									+ "\" not added because not found in script.");
						}
						else if (obj.getIndex() == lastIndex) {
							LinerUtil.updateDataLines(allData);
							update();
							verticalScrollBar.setValue(verticalScrollBar.getMaximum());
							form.update(PaneUpdate.REDRAW);
							JOptionPane.showMessageDialog(null, "Object " + obj.getName() 
									+ " added at index " + obj.getIndex());
						}
						else {
							JOptionPane.showMessageDialog(null, "Object " + obj.getName() 
									+ " not added because it already exists at index " + obj.getIndex());
						}
						addText.setText("");
					}
					}
				);
		
		add(bottomPane, BorderLayout.SOUTH);
		
		
		setSize(400,800);
		setLocationRelativeTo(form);
		if (openedElsewhere) {
			setDefaultCloseOperation(DISPOSE_ON_CLOSE);
			addWindowListener(new java.awt.event.WindowAdapter() {
			    @Override
			    public void windowClosing(java.awt.event.WindowEvent windowEvent) {
			        form.dispose();
			    }
			});
		}
		else {
			setDefaultCloseOperation(EXIT_ON_CLOSE);
		}
	}
	
	public void update() {
		listPane.removeAll();
		objectLines.clear();
		int i = 0;
		for (ScriptObject object: script.getObjects()) {
			ObjectLine line = new ObjectLine(object, script, i);
			objectLines.add(line);
			listPane.add(line);
			i++;
		}
		revalidate();
	}

	public void updateScript(Script script) {
		for (ObjectLine line: objectLines) {
			ScriptObject obj = line.getObject();
			int index = line.getObjectType().getSelectedIndex();
			ScriptObjectType newValue = ScriptObjectType.values()[(index)];
			if (newValue.equals(ScriptObjectType.Repeat) && !obj.getType().equals(ScriptObjectType.Repeat)) {
				int index2 = line.getRepeatOf().getSelectedIndex();
				ScriptObject obj2 = script.getObjects().get(index2);
				ScriptObject holder1;
				ScriptObject holder2;
				if (obj2.getScenes().size() > obj.getScenes().size() 
						|| obj2.getType().equals(ScriptObjectType.Repeat)) {
					holder1 = obj;
					holder2 = obj2;
				}
				else {
					holder1 = obj2;
					holder2 = obj;
				}
	
				if (!holder1.getType().equals(ScriptObjectType.Unknown) &&
						!holder2.getType().equals(ScriptObjectType.Repeat))
					holder2.setType(holder1.getType());
				holder1.setType(newValue);
				if (holder2.getType().equals(ScriptObjectType.Repeat))
					holder1.setRepeatOf(holder2.getRepeatOf());
				else
					holder1.setRepeatOf(holder2.getIndex());
				if (!holder2.getType().equals(ScriptObjectType.Repeat))
					holder2.setRepeatOf(null);
			}
			else
				obj.setType(newValue);
		}
		update();
		form.update();
	}
}
