package liner;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import util.FileUtil;

import com.fasterxml.jackson.core.JsonProcessingException;

import domain.AllData;

public class ScriptForm extends JFrame
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	AllData allData;
	
	FormValues values = new FormValues();
	final HelpForm help = new HelpForm(values);
	boolean helpVisible = false;
	
	public ObjectForm form;
		
	public void setObjectForm(ObjectForm form) {
		this.form = form;
	}

	boolean altPressed = false;
	boolean ctrlPressed = false;
	boolean shftPressed = false;
	
	public boolean isAltPressed() {
		return altPressed;
	}
	
	public boolean isCtrlPressed() {
		return ctrlPressed;
	}
	
	public boolean isShftPressed() {
		return shftPressed;
	}
	
	List<LineData> dataLines;
	int totalPages;
	int linesPerPage;
	int scriptLength;
	int lineLength;
	
	int curPage;
	int curLine;
	int curLineOnPage;
	
	List<JLine> list;
	JLine header;
	JPanel listPane;
	JScrollPane listScroller;
	JScrollBar verticalScrollBar;
	
	JPanel bottomPane;
	JButton first;
	JButton previous;
	JButton next;
	JButton last;
	JLabel page;
	JTextField pageNum;
	JLabel ofPage;
	JLabel sub;
	JButton saveJson;
	JButton saveZip;
	
	Timer timer = new Timer();
	boolean paused = true;
	
	public class MyKeyListener implements KeyListener {
		
		int counter = 0;
		@Override
		public void keyTyped(KeyEvent e) {
		}

		@Override
		public void keyPressed(KeyEvent e) {
			if (e.getKeyCode()== values.getAlt()) {
				altPressed = true;
			}
			if (e.getKeyCode()==values.getCtrl()) {
				ctrlPressed = true;
			}
			if (e.getKeyCode()==values.getShft()) {
				shftPressed = true;
			}
			counter++;
			if (counter > 20)
				processKey(e);
		}

		@Override
		public void keyReleased(KeyEvent e) {
			//resets
			counter = 0;
			if (e.getKeyCode()== values.getAlt()) {
				altPressed = false;
			}
			if (e.getKeyCode()==values.getCtrl()) {
				ctrlPressed = false;
			}
			if (e.getKeyCode()==values.getShft()) {
				shftPressed = false;
			}
			
			System.out.println(KeyEvent.getKeyText(e.getKeyCode()));
			System.out.println(e.getKeyCode());
			
			//need to be released to work
			if (e.getKeyCode()== values.getFirstPage()) {
				firstPage();
			}
			if (e.getKeyCode()== values.getLastPage()) {
				lastPage();
			}
			if (e.getKeyCode()== values.getFirstLine()) {
				selectLineOnPage(0);
			}
			if (e.getKeyCode()==values.getLastLine()) {
				selectLineOnPage(linesPerPage-1);
			}
			if (e.getKeyCode()==values.getHelp()) {
				helpVisible = !helpVisible;
				help.setVisible(helpVisible);
			}
			if (e.getKeyCode()==values.getToggleCut()) {
				list.get(curLineOnPage).toggleNewShot();
				if(altPressed) 
					list.get(curLineOnPage).copyPreviousCut(1);
				else if(ctrlPressed)
					list.get(curLineOnPage).copyPreviousCut(2);
			}
			if (e.getKeyCode()==values.getToggleObj1()) {
				list.get(curLineOnPage).toggleCharacter(0);
			}
			if (e.getKeyCode()==values.getToggleObj2()) {
				list.get(curLineOnPage).toggleCharacter(1);
			}
			if (e.getKeyCode()==values.getToggleObj3()) {
				list.get(curLineOnPage).toggleCharacter(2);
			}
			if (e.getKeyCode()==values.getToggleObj4()) {
				list.get(curLineOnPage).toggleCharacter(3);
			}
			if (e.getKeyCode()==values.getToggleObj5()) {
				list.get(curLineOnPage).toggleCharacter(4);
			}
			if (e.getKeyCode()==values.getToggleObj6()) {
				list.get(curLineOnPage).toggleCharacter(5);
			}
			if (e.getKeyCode()==values.getToggleObj7()) {
				list.get(curLineOnPage).toggleCharacter(6);
			}
			if (e.getKeyCode()==values.getSave() && ctrlPressed) {
				saveZip(false);
			}
			if (e.getKeyCode()==values.getPlayPause()) {
				if (paused) {
					timer = new Timer();
					timer.schedule(new AutoAdvance(), 0, values.getSpeed());
				}
				else {
					timer.cancel();
				}
				paused = !paused;
			}
			processKey(e);
		}

	}
	
	private void processKey(KeyEvent e) {
		if (e.getKeyCode()==values.getNextPage()) {
			nextPage();
		}
		if (e.getKeyCode()==values.getPrevPage()) {
			previousPage();
		}
		if (e.getKeyCode()==values.getLineDown()) {
			nextLine();
		}
		if (e.getKeyCode()==values.getLineUp()) {
			previousLine();
		}
		if (e.getKeyCode()==values.getShotUp()) {
			list.get(curLineOnPage).advanceShotType(false);
		}
		if (e.getKeyCode()==values.getShotDown()) {
			list.get(curLineOnPage).advanceShotType(true);
		}
		if (e.getKeyCode()==values.getCleanUp()) {
			list.get(curLineOnPage).advanceCleanType(false);
		}
		if (e.getKeyCode()==values.getCleanDown()) {
			list.get(curLineOnPage).advanceCleanType(true);
		}
		if (e.getKeyCode()==values.getMotionUp()) {
			list.get(curLineOnPage).advanceMotion(false);
		}
		if (e.getKeyCode()==values.getMotionDown()) {
			list.get(curLineOnPage).advanceMotion(true);
		}
	}
	
	public ScriptForm(AllData all, int linesPP)	{
		this(all, linesPP, false);
	}
	
	public ScriptForm(AllData all, int linesPP, boolean openedElsewhere)
	{
		setTitle("Script Liner Tool: " + all.getScript().getFileName());
		allData = all;
		dataLines = all.getDataLines();
		curPage = 0;
		curLine = 0;
		linesPerPage = linesPP;
		lineLength = all.getMaxLength();
		scriptLength = dataLines.size();
		totalPages = scriptLength/linesPerPage;
		
		list = new ArrayList<JLine>();
		listPane = new JPanel();
		listPane.setLayout(new BoxLayout(listPane, BoxLayout.Y_AXIS));
		
		//add the lines
		System.out.println("length " + lineLength);
		header = new JLine(this, dataLines, lineLength, allData.getScript(), values); //, this);
		header.setBackground(values.getHeaderColor());
		listPane.add(header);
		for (int i = 0; i < linesPerPage; i++) {
			JLine line = new JLine(this, dataLines, lineLength, allData.getScript(), values); //, this);
			final int toLine = i;
			line.addMouseListener(
					new MouseAdapter(){
						@Override
						public void mousePressed(MouseEvent e){
						}
						@Override
						public void mouseReleased(MouseEvent e) {
							if (altPressed) {
								copyLineOnPage(toLine);
							}
							else
								selectLineOnPage(toLine);
						}
					}
					);
			list.add(line);
			listPane.add(list.get(i));
		}
		list.get(0).setCur(true);
		updateLines();
		listScroller = new JScrollPane(listPane);
		verticalScrollBar = listScroller.getVerticalScrollBar();
		verticalScrollBar.setUnitIncrement(45);
		
        listScroller.setAlignmentX(LEFT_ALIGNMENT);
		add(listScroller, BorderLayout.CENTER);
		
		//bottom pane
		bottomPane = new JPanel();
		first = new JButton("First");
		first.setEnabled(false);
		first.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						firstPage();
					}
				}
				);
		last = new JButton("Last");
		last.setEnabled(totalPages>curPage);
		last.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						lastPage();
					}
				}
				);
		previous = new JButton("Previous");
		previous.setEnabled(false);
		previous.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						previousPage();
					}

				}
				);
		next = new JButton("Next");
		next.setEnabled(totalPages>curPage);
		next.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						nextPage();
					}
				}
				);
		page = new JLabel("Page");
		pageNum = new JTextField("" + curPage, 2);
		pageNum.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						jumpToPage(Integer.parseInt(pageNum.getText()));
					}
				}
				);
		ofPage = new JLabel("of " + totalPages);
		
		sub = new JLabel(" Save data for " + allData.getScript().getFileName() + ": ");
		saveJson =new JButton();
		saveJson.setText("Save to JSON");
		saveJson.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						saveJson(true);
						requestFocus();
					}
				}
				);
		saveZip =new JButton();
		saveZip.setText("Save to Zip");
		saveZip.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						saveZip(true);
						requestFocus();
					}
				}
				);
		
		bottomPane.add(first);
		bottomPane.add(previous);
		
		bottomPane.add(page);
		bottomPane.add(pageNum);
		bottomPane.add(ofPage);
		
		bottomPane.add(next);
		bottomPane.add(last);
		
		bottomPane.add(sub);
		bottomPane.add(saveJson);
		bottomPane.add(saveZip);
		
		add(bottomPane, BorderLayout.SOUTH);
		
		KeyListener listener = new MyKeyListener();
		addKeyListener(listener);
		setFocusable(true);
		
		setSize(1350,800);
		setLocationRelativeTo(null);
		if (openedElsewhere) {
			setDefaultCloseOperation(DISPOSE_ON_CLOSE);
			addWindowListener(new java.awt.event.WindowAdapter() {
			    @Override
			    public void windowClosing(java.awt.event.WindowEvent windowEvent) {
			        form.dispose();
			    }
			});
		}
		else {
			setDefaultCloseOperation(EXIT_ON_CLOSE);
		}
	}

	private void saveJson(boolean display) {
		String filename = allData.getScript().getFileName() + ".json";
		String output;
		try {
			output = FileUtil.toJsonString(allData);
		} catch (JsonProcessingException e1) {
			output = "whoops";
			throw new RuntimeException("Trouble parsing the data to json.");
		}
		try {
			FileUtil.writeFile(filename, 
					output);
			if (display)
				JOptionPane.showMessageDialog(null, "Data saved to " 
						+ filename);
		} catch (FileNotFoundException e1) {
			throw new RuntimeException("Could not write data to file");
		}
	}
	
	private void saveZip(boolean display) {
		String baseFilename = allData.getScript().getFileName();
		String output;
		try {
			output = FileUtil.toJsonString(allData);
		} catch (JsonProcessingException e1) {
			output = "whoops";
			throw new RuntimeException("Trouble parsing the data to json.");
		}
		try {
			FileUtil.writeZipFile(baseFilename, 
					output);
			if (display)
				JOptionPane.showMessageDialog(null, "Data written to " 
						+ baseFilename +".zip");
		} catch (Exception e1) {
			throw new RuntimeException("Could not write data to file");
		}
	}
	
	/**
	 * Updates the script form with default of redraw
	 */
	public void update() {
		update(PaneUpdate.REDRAW);
	}
	
	/**
	 * Updates the script form with PaneUpdate value
	 * @param value the type of update
	 */
	public void update(PaneUpdate value) {
		pageNum.setText("" + curPage);
		requestFocus();
		if (!previous.isEnabled() && curPage > 0)
			previous.setEnabled(true);
		if (previous.isEnabled() && curPage == 0) {
			previous.setEnabled(false);
		}
		if (!next.isEnabled() && curPage < totalPages)
			next.setEnabled(true);
		if (next.isEnabled() && curPage == totalPages)
			next.setEnabled(false);
		if (!first.isEnabled() && curPage > 0)
			first.setEnabled(true);
		if (first.isEnabled() && curPage == 0)
			first.setEnabled(false);
		if (!last.isEnabled() && curPage < totalPages)
			last.setEnabled(true);
		if (last.isEnabled() && curPage == totalPages)
			last.setEnabled(false);
		updateLines();
		if (value == PaneUpdate.NEXT || value == PaneUpdate.FIRST)
			verticalScrollBar.setValue(verticalScrollBar.getMinimum());
		else if (value == PaneUpdate.PREVIOUS)
			verticalScrollBar.setValue(verticalScrollBar.getMaximum());
	}
	
	/**
	 * Updates all lines on the page
	 */
	public void updateLines() {
		if (curPage > 0) {
			int back = curPage * linesPerPage;
			while (dataLines.get(back).getSceneNum()==null)
				back--;
			header.updateHeader(dataLines.get(back));
		}
		else
			header.setOpener(allData.getScript().getFileName());
		
		int cur; 
		for (int i = 0; i < linesPerPage; i++) {
			list.get(i).setActive(false);
			cur = curPage * linesPerPage + i;
			if (cur < scriptLength)
				list.get(i).update(dataLines.get(cur));
			else
				list.get(i).updateBlank();
			list.get(i).setActive(true);
		}
	}
	
	private void nextPage() {
		if (curPage < totalPages) {
			curPage++;
			update(PaneUpdate.NEXT);
		}
		updateCurLineTop();
		requestFocus();
	}
	
	private void previousPage() {
		if (curPage > 0) {
			curPage--;
			update(PaneUpdate.PREVIOUS);
		}
		updateCurLineBottom();
		requestFocus();
	}
	
	private void lastPage() {
		curPage=totalPages;
		update(PaneUpdate.LAST);
		updateCurLineTop();
		requestFocus();
	}
	
	private void firstPage() {
		curPage=0;
		update(PaneUpdate.FIRST);
		updateCurLineTop();
		requestFocus();
	}
	
	private void jumpToPage(int page) {
		int newPage = page;
		if (newPage >= 0 && newPage <= totalPages) {
			curPage = newPage;
			update(PaneUpdate.OTHER);
		}
		else
			pageNum.setText("" + curPage);
		updateCurLineTop();
		requestFocus();
	}
	
	private void updateCurLineTop() {
		list.get(curLineOnPage).setCur(false);
		curLine = curPage * linesPerPage;
		curLineOnPage = 0;
		list.get(curLineOnPage).setCur(true);
	}
	
	private void updateCurLineBottom() {
		list.get(curLineOnPage).setCur(false);
		curLine = ((curPage+1) * linesPerPage) - 1;
		curLineOnPage = linesPerPage-1;
		list.get(curLineOnPage).setCur(true);
	}
	
	private void nextLine() {
		if (curLine+1 < scriptLength) {
			curLine++;
			list.get(curLineOnPage).setCur(false);
			if ((curLineOnPage+1) >= linesPerPage) {
				curLineOnPage = 0;
				nextPage();
			}
			else {
				curLineOnPage++;
				setVerticalScrollBar(true);
			}
			list.get(curLineOnPage).setCur(true);
		}
	}
	
	private void previousLine() {
		if (curLine > 0) {
			curLine--;
			list.get(curLineOnPage).setCur(false);
			if ((curLineOnPage <= 0)) {
				curLineOnPage = linesPerPage - 1;
				previousPage();
			}
			else {
				curLineOnPage--;
				setVerticalScrollBar(false);
			}
			list.get(curLineOnPage).setCur(true);
		}
	}

	private void setVerticalScrollBar(boolean down) {
		int moveAmount = verticalScrollBar.getMaximum()/linesPerPage;
		int curHeight = listScroller.getHeight(); // this.getHeight();
		int curBar = verticalScrollBar.getValue();
		int positioning = curLineOnPage * verticalScrollBar.getMaximum() / linesPerPage;
		int cutOff;
		if (curPage == 0)
			cutOff = curBar + curHeight - (1 * moveAmount);
		else
			cutOff = curBar + curHeight - (2 * moveAmount);
		System.out.println(curBar);
		System.out.println(positioning);
		System.out.println(cutOff);
		if (!(positioning >= curBar && positioning <= cutOff)) {
			if (down)
				if (curPage == 0)
					verticalScrollBar.setValue(curBar + moveAmount + 1);
				else
					verticalScrollBar.setValue(curBar + moveAmount);
			else
				if( !(curLineOnPage >= linesPerPage - 2))
					verticalScrollBar.setValue(curBar - moveAmount - 1);
		}
	}
	
	private void selectLineOnPage(int line) {
		list.get(curLineOnPage).setCur(false);
		curLineOnPage = line;
		curLine = curPage * linesPerPage + curLineOnPage;
		list.get(curLineOnPage).setCur(true);
		requestFocus();
	}
	
	private void copyLineOnPage(int line) {
		list.get(curLineOnPage).copyJLine(list.get(line));
		requestFocus();
	}
	
	class AutoAdvance extends TimerTask {
		public void run() {
			nextLine();
		}
	}

}


