package lister;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileNameExtensionFilter;

import com.fasterxml.jackson.core.JsonProcessingException;

import util.FileUtil;

/**
 * The feature form for changing feature settings
 * 
 * @author David Smith
 *
 */
public class FeatureForm extends JFrame {
	
	FeatureSettings settings;
	List<JPanel> featurePanels = new ArrayList<JPanel>();
	JPanel sPane = new JPanel();
	
	public FeatureForm(FeatureSettings settings) {
		this.settings = settings;
		setTitle("Customize Feature Settings");
		createForm();
	}
	
	private void createForm() {
		JPanel pane = new JPanel();
		pane.setLayout(new FlowLayout());
		for (final TargetFeature tF: TargetFeature.values()) {
			
			JPanel tFPane = new JPanel();
			tFPane.setLayout(new BoxLayout(tFPane, BoxLayout.Y_AXIS));
			JLabel tFLabel = new JLabel(tF.getName());
			tFPane.add(tFLabel);
			for (final Features f: Features.values()) {
				JPanel fPane = new JPanel();
				fPane.setLayout(new FlowLayout());
				JLabel fLabel = new JLabel(f.name());
				String tip = f.getName();
				if (f.isPure()) {
					fLabel.setForeground(Color.BLUE);
					tip = tip + " - pure";
				}
				else {
					tip = tip + " - nonpure";
				}
				fLabel.setToolTipText(tip);
				final JTextField fField = new JTextField("XXXX");
				final JCheckBox fBox = new JCheckBox();
				fBox.setSelected(settings.isUse(tF, f));
				Dimension d = fField.getPreferredSize();
				fField.setPreferredSize(new Dimension(d.width, d.height));
				fField.setText(settings.getWeight(tF,f) + "");
				
				fBox.addActionListener(
						new ActionListener(){
							@Override
							public void actionPerformed(
									ActionEvent e){
								settings.setUse(tF, f, fBox.isSelected());
							}
						}
						);
				fField.addActionListener(
						new ActionListener(){
							@Override
							public void actionPerformed(
									ActionEvent e){
								String value = fField.getText();
								double weight;
								try {
									weight = Double.parseDouble(value);
								}
								catch (Exception ex) {
									JOptionPane.showMessageDialog(null, "Weight of " +tF.getName() + ", " + f.name()
											+ " must be a number.");
									return;
								}
								settings.setWeight(tF, f, weight);
							}
						});
				fPane.add(fLabel);
				fPane.add(fBox);
				fPane.add(fField);
				tFPane.add(fPane);
			}
			pane.add(tFPane);
		}
		
		JPanel buttonPane = new JPanel();
		JButton useSettings = new JButton("Use Settings");
		JButton loadSettings = new JButton("Load Settings");
		JButton saveSettings = new JButton("Save Settings");
		
		useSettings.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						dispose();
					}
				}
				);
		saveSettings.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						File file = getSaveFile();
						if (file!=null) {
							String data = "";
							try {
								data = FileUtil.toJsonString(settings);
							} catch (JsonProcessingException e1) {
								JOptionPane.showMessageDialog(null, "Unable to process file to json string.");
							}
							try {
								FileUtil.writeFile(file.getAbsolutePath().toString(), data);
							} catch (FileNotFoundException e1) {
								JOptionPane.showMessageDialog(null, "Unable to write setting data to file.");
							}
						}
					}
				}
				);
		loadSettings.addActionListener(
				new ActionListener(){
					@Override
					public void actionPerformed(
							ActionEvent e){
						File file = getOpenFile();
						settings.copySettings(FileUtil.getFeatureSettingsFromFile(file.getAbsolutePath().toString()));
						sPane = new JPanel();
						createForm();
					}
				}
				);
		buttonPane.setLayout(new FlowLayout());
		buttonPane.add(useSettings);
		buttonPane.add(loadSettings);
		buttonPane.add(saveSettings);
		sPane.setLayout(new BoxLayout(sPane, BoxLayout.Y_AXIS));
		sPane.add(pane);
		sPane.add(buttonPane);
		add(sPane);
		setSize(1000,1000);
		setLocationRelativeTo(null);
		setVisible(true);
//		setDefaultCloseOperation(EXIT_ON_CLOSE);
	}
	
	class FeaturePanel extends JPanel {
		
	}
	
	private File getSaveFile() {
		JButton open = new JButton();
		JFileChooser fc = new JFileChooser(){
		    @Override
		    public void approveSelection(){
		        File f = getSelectedFile();
		        if(f.exists()){
		            int result = JOptionPane.showConfirmDialog(this,"The file already exists. Overwrite?","File Exists", 
		            		JOptionPane.YES_NO_OPTION);
		            switch(result){
		                case JOptionPane.YES_OPTION:
		                    super.approveSelection();
		                    return;
		                case JOptionPane.NO_OPTION:
		                    return;
//		                case JOptionPane.CLOSED_OPTION:
//		                    return;
		                case JOptionPane.CANCEL_OPTION:
		                    cancelSelection();
		                    return;
		            }
		        }
		        super.approveSelection();
		    }        
		};
		FileNameExtensionFilter filter = new FileNameExtensionFilter("JSON Files", "json");
		fc.setFileFilter(filter);
		fc.setCurrentDirectory(new java.io.File("."));
		fc.setDialogTitle("Save Settings File");
		fc.setApproveButtonText("Save");
		if (fc.showOpenDialog(open) == JFileChooser.APPROVE_OPTION) {
			
		}
		fc.getCurrentDirectory();
		return fc.getSelectedFile();
	}
	
	private File getOpenFile() {
		JButton open = new JButton();
		JFileChooser fc = new JFileChooser();
		fc.setCurrentDirectory(new java.io.File("."));
		FileNameExtensionFilter filter = new FileNameExtensionFilter("JSON Files", "json");
		fc.setFileFilter(filter);
		fc.setDialogTitle("Load Settings File");
		fc.setApproveButtonText("Open");
		if (fc.showOpenDialog(open) == JFileChooser.APPROVE_OPTION) {
			fc.getCurrentDirectory();
			return fc.getSelectedFile();
		}
		else 
			return null;
	}
}
