package lister;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class FeatureSettings {
	
	HashMap<String, HashMap<String, FeatureSetting>> targetFeatures;
	
	public FeatureSettings() {
		targetFeatures = new HashMap<String, HashMap<String, FeatureSetting>>();
		for (TargetFeature tF: TargetFeature.values()) {
			HashMap<String, FeatureSetting> settings = new HashMap<String, FeatureSetting>();
			for (Features f: Features.values()) {
				FeatureSetting setting = new FeatureSetting();
				setting.setUse(true);
				setting.setWeight(1.0);
				settings.put(f.name(), setting);
			}
			targetFeatures.put(tF.getName(), settings);
		}
		setDefaults();
	}
	
	public void setDefaults() {
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.cut.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.shotType.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.cleanType.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.motions.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.linesSinceShotTypeChange.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.lastShotTypeNoCut.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.lastShotTypeWhenCut.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.linesSinceCleanChange.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.lastCleanType.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.linesSinceMotionChange.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.lastMotionType.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.tiltCount.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.zoomCount.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.panCount.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.dialogueCountInShot.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.uniqueDialogueCountInShot.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.actionCountInShot.name()).setUse(false);
		targetFeatures.get(TargetFeature.Cut.name()).get(Features.lineCountInShot.name()).setUse(false);
		
		targetFeatures.get(TargetFeature.ShotType.name()).get(Features.shotType.name()).setUse(false);
		targetFeatures.get(TargetFeature.ShotType.name()).get(Features.cleanType.name()).setUse(false);
		targetFeatures.get(TargetFeature.ShotType.name()).get(Features.motions.name()).setUse(false);
		targetFeatures.get(TargetFeature.ShotType.name()).get(Features.linesSinceCleanChange.name()).setUse(false);
		targetFeatures.get(TargetFeature.ShotType.name()).get(Features.lastCleanType.name()).setUse(false);
		targetFeatures.get(TargetFeature.ShotType.name()).get(Features.linesSinceMotionChange.name()).setUse(false);
		targetFeatures.get(TargetFeature.ShotType.name()).get(Features.lastMotionType.name()).setUse(false);
		targetFeatures.get(TargetFeature.ShotType.name()).get(Features.tiltCount.name()).setUse(false);
		targetFeatures.get(TargetFeature.ShotType.name()).get(Features.zoomCount.name()).setUse(false);
		targetFeatures.get(TargetFeature.ShotType.name()).get(Features.panCount.name()).setUse(false);
		
		targetFeatures.get(TargetFeature.CleanType.name()).get(Features.cleanType.name()).setUse(false);
		targetFeatures.get(TargetFeature.CleanType.name()).get(Features.motions.name()).setUse(false);
		targetFeatures.get(TargetFeature.CleanType.name()).get(Features.linesSinceMotionChange.name()).setUse(false);
		targetFeatures.get(TargetFeature.CleanType.name()).get(Features.lastMotionType.name()).setUse(false);
		targetFeatures.get(TargetFeature.CleanType.name()).get(Features.tiltCount.name()).setUse(false);
		targetFeatures.get(TargetFeature.CleanType.name()).get(Features.zoomCount.name()).setUse(false);
		targetFeatures.get(TargetFeature.CleanType.name()).get(Features.panCount.name()).setUse(false);
		
		targetFeatures.get(TargetFeature.Motion.name()).get(Features.motions.name()).setUse(false);
		
		targetFeatures.get(TargetFeature.CleanType.name()).get(Features.shotType.name()).setWeight(5.0);
		
	}
	
	public void copySettings(FeatureSettings settings) {
		for (TargetFeature tF: TargetFeature.values()) {
			for (Features f: Features.values()) {
				targetFeatures.get(tF.name()).get(f.name()).setUse(settings.isUse(tF,f));
				targetFeatures.get(tF.name()).get(f.name()).setWeight(settings.getWeight(tF, f));
			}
		}
	}
	
	public HashMap<String, HashMap<String, FeatureSetting>> getTargetFeatures() {
		return targetFeatures;
	}

	public void setTargetFeatures(HashMap<String, HashMap<String, FeatureSetting>> targetFeatures) {
		this.targetFeatures = targetFeatures;
	}
	
	public boolean isUse(TargetFeature tF, Features f) {
		return targetFeatures.get(tF.getName()).get(f.name()).isUse();
	}
	public void setUse(TargetFeature tF, Features f, boolean use) {
		targetFeatures.get(tF.getName()).get(f.name()).setUse(use);
	}
	public double getWeight(TargetFeature tF, Features f) {
		return targetFeatures.get(tF.getName()).get(f.name()).getWeight();
	}
	public void setWeight(TargetFeature tF, Features f, double weight) {
		targetFeatures.get(tF.getName()).get(f.name()).setWeight(weight);
	}
	
}
