package lister;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonIgnore;

import domain.CleanType;
import domain.CutOptions;
import domain.IntExt;
import domain.Motion;
import domain.ShotType;
import domain.SceneObjectType;

public class Vector {
	
	public TargetFeature target_feature;
	public Option[] optionTotals;
	public Feature[] feature;
	
	private List<String> maxIntAsStrings(int max) {
		List<String> strings = new ArrayList<String>();
		for (int i = 0; i < max; i ++)
			strings.add(i + "");
		return strings;
	}
	
	public Vector() {
	}
	
	public Vector(TargetFeature name, List<String> optionNames) {
		this.target_feature = name;
		
		optionTotals = new Option[optionNames.size()];
		for (int i = 0; i <optionNames.size(); i++)
			optionTotals[i] = new Option(optionNames.get(i));
		
		List<String> selections = null;
		
		feature = new Feature[Features.values().length];
		for (Features f: Features.values()) {
			switch (f) {
			case cut:
				selections = CutOptions.getStrings();
				break;
			case shotType:
				selections = ShotType.getValues();
				break;
			case cleanType:
				selections = CleanType.getValues();
				break;
			case motions:
				selections = Motion.getStrings();
				break;
			case linesSinceCut:
				selections = maxIntAsStrings(Constants.MAXLINESSINCECUT);
				break;
			case linesSinceShotTypeChange:
				selections = maxIntAsStrings(Constants.MAXLINESSINCESHOTTYPECHANGE);
				break;
			case lastShotTypeNoCut:
				selections = ShotType.getValues();
				break;
			case lastShotTypeWhenCut:
				selections = ShotType.getValues();
				break;
			case linesSinceCleanChange:
				selections = maxIntAsStrings(Constants.MAXLINESSINCECLEANCHANGE);
				break;
			case lastCleanType:
				selections = Motion.getStrings();
				break;
			case linesSinceMotionChange:
				selections = maxIntAsStrings(Constants.MAXLINESSINCEMOTIONCHANGE);
				break;
			case lastMotionType:
				selections = Motion.getStrings();
				break;
			case tiltCount:
				selections = maxIntAsStrings(Constants.MAXMOTIONCOUNT);
				break;
			case zoomCount:
				selections = maxIntAsStrings(Constants.MAXMOTIONCOUNT);
				break;
			case panCount:
				selections = maxIntAsStrings(Constants.MAXMOTIONCOUNT);
				break;
			case dialogueCountInShot:
				selections = maxIntAsStrings(Constants.MAXSCENEOBJECTS);
				break;
			case uniqueDialogueCountInShot:
				selections = maxIntAsStrings(Constants.MAXSCENEOBJECTS);
				break;
			case actionCountInShot:
				selections = maxIntAsStrings(Constants.MAXSCENEOBJECTS);
				break;
			case lineCountInShot:
				selections = maxIntAsStrings(Constants.MAXLINECOUNTINSHOT);
				break;
			case sceneObjectType:
				selections = SceneObjectType.getStrings();
				break;
			case linesSinceObjectChange:
				selections = maxIntAsStrings(Constants.MAXLINESSINCEOBJECTCHANGE);
				break;
			case linesSinceNewScene:
				selections = maxIntAsStrings(Constants.MAXLINESSINCENEWSCENE);
				break;
			case intExt:
				selections = IntExt.getStrings();
				break;
			case scriptObjectsInScene:
				selections = maxIntAsStrings(Constants.MAXSCRIPTOBJECTSINSCENE);
				break;
			case dialogueBlocksInScene:
				selections = maxIntAsStrings(Constants.MAXSCENEOBJECTS);
				break;
			case actionBlocksInScene:
				selections = maxIntAsStrings(Constants.MAXSCENEOBJECTS);
				break;
			case sceneLength:
				selections = maxIntAsStrings(Constants.MAXSCENELENGTH);
				break;
			default:
			;
			}
			feature[f.ordinal()] =
					new Feature(selections, optionNames, f);
		}
	}
		
	public static class Feature {
		public Features feature;
		public Selection[] selection;
		
		public Feature() {
		}
		
		public Feature(List<String> values, List<String> options, Features name) {
			int length = values.size();
			this.feature = name;
			selection = new Selection[length];
			for (int i = 0; i < length; i++) {
				selection[i] = new Selection(options, values.get(i));
			}
			
		}
		
		public int totalSelectionGivenOption(int selPick, int optionPick) {
			return selection[selPick].option[optionPick].count;
		}
	}
	
	public static class Selection {
		public String selected;
		public Option[] option;
		
		public Selection() {
		}
		
		public Selection(List<String> options, String selected) {
			this.selected = selected;
			option = new Option[options.size()];
			for (int i = 0; i < options.size(); i++) {
				option[i] = new Option(options.get(i));
			}
		}
		
		@JsonIgnore
		public int getTotal() {
			int total = 0;
			for (Option o: option)
				total+=o.count;
			return total;
		}
		
	}
	
	public static class Option {
		public String pick;
		public int count;
		
		public Option() {
		}
		
		public Option(String selected) {
			this.pick = selected;
			count = 0;
		}
	}
	
	public int totalOption(int optionPick) {
		int total = 0;
		for (Feature f : feature) {
			for (Selection s: f.selection) {
				total += s.option[optionPick].count;
			}
		}
		return total;
	}
	
	/**
	 * Given an option picked, this will return the probability of a picked Selection.
	 * This is calculated by returning the number of times that option was picked under the given selection
	 * divided by the total number of times that option was picked.
	 * @param feat The Feature being analyzed
	 * @param selPick The Selection picked
	 * @param optionPick The option picked
	 * @return the probability of the picked selection given the picked option
	 */
	public Fraction probabilitySelectionGivenOption(int feat, int selPick, int optionPick) {
		Fraction prob;
		try {
			int selectionCountForOption = feature[feat].selection[selPick].option[optionPick].count;
			int optionTotalOfPick = optionTotals[optionPick].count;
			prob = new Fraction(selectionCountForOption + Constants.LAPLACE,
//					optionTotalOfPick + (Constants.LAPLACE * optionTotals.length));
					optionTotalOfPick + (Constants.LAPLACE * feature[feat].selection.length));
		} catch (Exception e) {
			System.out.println("exception: " + target_feature); 
			System.out.print(feature[feat].feature + " "); 
			return null;
		}
		return prob;
	}
	
	/**
	 * Returns the probability of an option pick.
	 * Calculated by the number of times that option was picked, over the total number of times
	 * any option was picked.
	 * @param optionPick An option pick
	 * @return the probability of that option pick
	 */
	public Fraction probabilityOption(int optionPick) {
		int total = 0;
		for (Option o:optionTotals)
			total += o.count;
		return new Fraction(optionTotals[optionPick].count,total);
	}
	
	/**
	 * Returns the option with the highest probability
	 * This is done with Naive Bayes where the argmax is picked from a series of products.
	 * Each product starts with the Probability of the option 
	 * then times the probability of each selection given the option
	 * @param sel an array where each value is the selection chosen for each feature
	 * @return the option based on Naive Bayes
	 */
	public int getOption (HashMap<String, Integer> sel, FeatureSettings settings, List<String> skipFeature) {
		int current = 0;
		double high = 0;
		String[][] probs =  new String[optionTotals.length][feature.length];
		for (int o = 0; o < this.optionTotals.length; o++) {
			
			double total = probabilityOption(o).dec();
			for (int f = 0; f < this.feature.length; f++) {
				if (settings.isUse(target_feature, feature[f].feature) && !skipFeature.contains(feature[f].feature.name())) {
					Fraction prob = probabilitySelectionGivenOption(f, sel.get(feature[f].feature.name()), o);
					double probOut = prob.dec();
					double weight = settings.getWeight(target_feature, feature[f].feature);
					probOut = Math.pow(probOut, weight);
					probs[o][f] = prob.toString();
					total *= probOut;
				}
			}
			
			if (total > high) {
				current = o;
				high = total;
			}
		}
		return current;
	}
	
	public String debugProbs(String[][] probs) {
		String out = target_feature.getName() + "\r\n";
		for (int o = 0; o < this.optionTotals.length; o++) {
			out += target_feature.getOptions().get(o) + ": " 
					+ probabilityOption(o) + " [";
			for (int f = 0; f < this.feature.length ; f++) {
				out += feature[f].feature.name() +":" + probs[o][f];
				if (f != this.feature.length - 1)
					out+= ", ";
			}
			out += "]\r\n";
		}
		return out;
	}
}
