package lister;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.JOptionPane;

import liner.LineData;
import liner.LinerUtil;
import parser.Parser;
import util.FileUtil;
import util.Util;

import com.fasterxml.jackson.core.JsonProcessingException;

import domain.AllData;
import domain.Dialogue;
import domain.IntExt;
import domain.Motion;
import domain.SceneObject;
import domain.SceneObjectType;
import domain.Script;

public class VectorUtil {
	
	/**
	 * Creates a brand new vector file with a vector for each Target Feature
	 * The vectors are initialized empty
	 * @return 
	 */
	public static VectorFile createAllVectors() {
		VectorFile vectors = new VectorFile();
		for (TargetFeature tF: TargetFeature.values()) 
			vectors.put(tF.name(), 
					new Vector(tF, tF.getOptions()));
		return vectors;
	}
	
	/**
	 * Adds all the data from a lined script to the vectorfile supplied
	 * @param data the lined script
	 * @param v the vector file to add the data to
	 * @return the vector file with statistics of lined script added to it
	 */
	public static VectorFile addAllDataToVectors(AllData data, VectorFile v) {
		boolean use = true;
		int selected = 0;
		
		HashMap<String, Integer> counters = createCounters(data);
		
		HashMap<String, Integer> choice = new HashMap<String, Integer>();
		for (LineData line: data.getDataLines()) {
			
			choice.put(TargetFeature.Cut.name(), ((line.isNewShot()) ? 1 : 0));
			choice.put(TargetFeature.ShotType.name(), line.getShotType());
			choice.put(TargetFeature.CleanType.name(), line.getCleanType());
			choice.put(TargetFeature.Motion.name(), line.getMotion());
			if (line.getSceneNum()!=null) {
				updateNewSceneCounters(data, counters, line);
			}
			if (line.isNewShot() || line.getLineNum() == 0) {
				updateShotCounts(data, counters, line.getLineNum());
			}
			
			for (Features f: Features.values()) {
				switch (f) {
					case cut:
						selected=choice.get(TargetFeature.Cut.name());
						break;
					case shotType:
						selected=choice.get(TargetFeature.ShotType.name());
						break;
					case cleanType:
						selected=choice.get(TargetFeature.CleanType.name());
						break;
					case motions:
						selected=choice.get(TargetFeature.Motion.name());
						break;
					case linesSinceCut:
						selected=counters.get("linesSinceCut");
						break;
					case linesSinceShotTypeChange:
						selected=counters.get("linesSinceShotTypeChange");
						break;
					case lastShotTypeNoCut:
						if (skippable(line))
							use = false;
						else {
							selected = data.getDataLines().get(line.getLineNum()-1).getShotType();
						}
						break;
					case lastShotTypeWhenCut:
						if (line.getLineNum() == 0 	|| !(line.isNewShot() || line.getSceneNum() != null)) {
							use = false;
						}
						else {
							selected = data.getDataLines().get(line.getLineNum()-1).getShotType();
						}
						break;
					case linesSinceCleanChange:
						selected=counters.get("linesSinceCleanChange");
						break;
					case lastCleanType:
						if (skippable(line))
							use = false;
						else {
							selected = data.getDataLines().get(line.getLineNum()-1).getCleanType();
						}
						break;
					case linesSinceMotionChange:
						selected=counters.get("linesSinceMotionChange");
						break;
					case lastMotionType:
						if (skippable(line))
							use = false;
						else {
							selected = data.getDataLines().get(line.getLineNum()-1).getMotion();
						}
						break;
					case tiltCount:
						selected=counters.get("tiltCount");
						break;
					case zoomCount:
						selected=counters.get("zoomCount");
						break;
					case panCount:
						selected=counters.get("panCount");
						break;
					case dialogueCountInShot:
						selected=counters.get("dialogueCountInShot");
						break;
					case uniqueDialogueCountInShot:
						selected=counters.get("uniqueDialogueCountInShot");
						break;
					case actionCountInShot:
						selected=counters.get("actionCountInShot");
						break;
					case lineCountInShot:
						selected=counters.get("lineCountInShot");
						break;
					case sceneObjectType:
						selected = getObjectTypeOnLine(line.getLineNum(), data);
						break;
					case linesSinceObjectChange:
						selected=counters.get("linesSinceObjectChange");
						break;
					case linesSinceNewScene:
						selected=counters.get("linesSinceNewScene");
						break;
					case intExt:
						selected = counters.get("ie");
						break;
					case scriptObjectsInScene:
						selected=counters.get("objCount");
						break;
					case sceneLength:
						selected=counters.get("sceneLength");
						break;
					default:
						;
				}
				
				//update the choice for the given feature in each target feature
				if (use) {
					for (TargetFeature tF: TargetFeature.values()) {
						try {
						v.get(tF.name())
							.feature[f.ordinal()]
							.selection[selected]
							.option[choice.get(tF.name())]
							.count++;
						} catch (Exception e) {
							System.out.println("f.ordinal: " + f.ordinal());
							System.out.println("selection: " + selected);
							System.out.println("option: " + choice.get(tF.name()));
							throw new RuntimeException("Didn't work");
						}
					}
				}
				else
					use = true;
			}
			
			//update the overall count of choices made on line
			for (TargetFeature tF: TargetFeature.values()) 
				v.get(tF.name()).optionTotals[choice.get(tF.name())].count ++;
			
			updateCounters(data, counters, line);
		}
		
		return v;	
	}
	
	/**
	 * Returns the integer value of a SceneObject type
	 * @param lineNum The line being checked
	 * @param data The parsed script data
	 * @return the SceneObjectType as an int
	 */
	private static int getObjectTypeOnLine(int lineNum, AllData data) {
		String sel = data.getScript().getSceneObjects().get(data.getScript().getLines()[lineNum]).getType();
		int type = 0;
		try {
			type = SceneObjectType.fromString(sel).ordinal();
		} catch (NullPointerException e) {
			JOptionPane.showMessageDialog(null, "FromStringSOT bad line " + lineNum);
		}
		return type;
	}
	
	/**
	 * Adds all data from the training file to the vector file
	 * Makes sure the trainingFile is valid before proceeding
	 * If the vectorFile already exists, it adds to it, otherwise it creates a new one.
	 * @param trainingFile the training File, a lined script
	 * @param vectorFile the file to save the vectorFile to
	 * @return whether the population was a success
	 */
	public static boolean populateVectorsFile(File trainingFile, String vectorFile) {
//		String vectorFile = "vectors.json";
		String fileType = FileUtil.getFileType(trainingFile.getName());
		
		AllData data = null;
		if (fileType.equals("zip")) {
			data = FileUtil.getAllDataFromZip(trainingFile.getAbsolutePath());
		}
		else if (fileType.equals("json")) {
			data = FileUtil.getAllDataFromJson(trainingFile.getAbsolutePath());
		}
		else {
			JOptionPane.showMessageDialog(null, "File must be of type .zip or .json");
			return false;
		}
		String fileName = FileUtil.getName(trainingFile.getName());
		
		VectorFile vectors = FileUtil.getVectorsFromFile(vectorFile, false);
		if (vectors.contains(fileName)) {
			JOptionPane.showMessageDialog(null, "File " + trainingFile.getName() + " already added to Vector.");
			return false;
		}
		else
			vectors.add(fileName);
		vectors = VectorUtil.addAllDataToVectors(data, vectors);
		
		String output;
		try {
			output = FileUtil.toJsonString(vectors);
		} catch (JsonProcessingException e1) {
			output = "whoops";
			throw new RuntimeException("Trouble parsing the vectors to json.");
		}
		try {
			FileUtil.writeFile(vectorFile, 
					output);
			return true;
		} catch (Exception e1) {
			throw new RuntimeException("Could not write data to file");
		}
	}
	
	/**
	 * Lines the script using the given vectorFile
	 * Writes 
	 * @param file The script as a text file to be lined
	 * @param out the output file of the lined script
	 * @param vectorFile the vector file
	 * @param settings the feature settings to use in lining
	 * @return the name of the zip file as a string
	 */
	public static String scriptFileToAllDataFile(String file, String out, String vectorFile, FeatureSettings settings) {
		String name = FileUtil.getName(file);
		VectorFile vectors = FileUtil.getVectorsFromFile(vectorFile, true);
		if (vectors == null) {
			return null;
		}
		
		AllData allData;
		ArrayList<String> text = FileUtil.readFile(file);
		Script script = Parser.textToScript(text, name);
		allData = LinerUtil.createDataLines(text, script);
		if(LinerUtil.recheckScriptObjects(allData))
			LinerUtil.updateDataLines(allData);
		
		allData = naiveBayes(allData, vectors, settings);
		
		String outName = out;
		if (outName.contains(".")) {
			outName = outName.split("\\.")[0];
		}
		
		try {
			String output = FileUtil.writeZipFile(outName, FileUtil.toJsonString(allData));
			String printable = FileUtil.getName(out) + ".zip";
			JOptionPane.showMessageDialog(null, "Shot list created and written to " 
					+ printable);
			
			return output;
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Couldn't write shots to file");
			return null;
		}
	}
	
	/**
	 * Uses Naive Bayes to apply choices for each Target Feature on to each
	 * line of the script.
	 * @param data A parsed but unlined script
	 * @param vectors The vectors being used to apply choices
	 * @return A now lined script
	 */
	private static AllData naiveBayes(AllData data, VectorFile vectors, FeatureSettings settings) {
		Vector cutVector = vectors.get(TargetFeature.Cut.name());
		Vector shotVector = vectors.get(TargetFeature.ShotType.name());
		Vector cleanVector = vectors.get(TargetFeature.CleanType.name());
		Vector motionVector = vectors.get(TargetFeature.CleanType.name());
		
		HashMap<String, Integer> counters = createCounters(data);

		for (TargetFeature tF: TargetFeature.values()) {
			for (LineData line:data.getDataLines()) {

				if (line.getSceneNum()!=null) {
					updateNewSceneCounters(data, counters, line);
				}
				if (tF.ordinal()>TargetFeature.Cut.ordinal() && (line.isNewShot() || line.getLineNum() == 0)) {
					updateShotCounts(data, counters, line.getLineNum());
				}

				HashMap<String, Integer> selections = new HashMap<String, Integer>();
				ArrayList<String> skipFeatures = new ArrayList<String>();

				for (Features f: Features.values()) {
					switch (f) {
					case cut:
						selections.put(Features.cut.name(), ((line.isNewShot()) ? 1 : 0));
						break;
					case shotType:
						selections.put(Features.shotType.name(), line.getShotType());
						break;
					case cleanType:
						selections.put(Features.cleanType.name(), line.getCleanType());
						break;
					case motions:
						selections.put(Features.motions.name(), line.getMotion());
						break;
					case linesSinceCut:
						selections.put(Features.linesSinceCut.name(), counters.get("linesSinceCut"));
						break;
					case linesSinceShotTypeChange:
						selections.put(Features.linesSinceShotTypeChange.name(), counters.get("linesSinceShotTypeChange"));
						break;
					case lastShotTypeNoCut:
						if (skippable(line)) { //skip if first line of script, scene or shot
							skipFeatures.add(Features.lastShotTypeNoCut.name());
						}
						else {
							selections.put(Features.lastShotTypeNoCut.name(), 
									data.getDataLines().get(line.getLineNum() - 1).getShotType());
						}
						break;
					case lastShotTypeWhenCut:
						if (line.getLineNum() == 0 	|| !(line.isNewShot() || line.getSceneNum() != null)) {
							skipFeatures.add(Features.lastShotTypeWhenCut.name());
						}
						else {
							selections.put(Features.lastShotTypeWhenCut.name(),
									data.getDataLines().get(line.getLineNum() - 1).getShotType());
						}
						break;
					case linesSinceCleanChange:
						selections.put(Features.linesSinceCleanChange.name(), counters.get("linesSinceCleanChange"));
						break;
					case lastCleanType:
						if (skippable(line)) { //skip if first line of script, scene or shot
							skipFeatures.add(Features.lastCleanType.name());
						}
						else {
							selections.put(Features.lastCleanType.name(), 
									data.getDataLines().get(line.getLineNum() - 1).getCleanType());
						}
						break;
					case linesSinceMotionChange:
						selections.put(Features.linesSinceMotionChange.name(), counters.get("linesSinceMotionChange"));
						break;
					case lastMotionType:
						if (skippable(line)) { //skip if first line of script, scene or shot
							skipFeatures.add(Features.lastMotionType.name());
						}
						else {
							selections.put(Features.lastMotionType.name(), 
									data.getDataLines().get(line.getLineNum() - 1).getMotion());
						}
						break;
					case tiltCount:
						selections.put(Features.tiltCount.name(), counters.get("tiltCount"));
						if (counters.get("tiltCount") == 0)
							skipFeatures.add(Features.tiltCount.name());
						break;
					case zoomCount:
						selections.put(Features.zoomCount.name(), counters.get("zoomCount"));
						if (counters.get("zoomCount") == 0)
							skipFeatures.add(Features.zoomCount.name());
						break;
					case panCount:
						selections.put(Features.panCount.name(), counters.get("panCount"));
						if (counters.get("panCount") == 0)
							skipFeatures.add(Features.panCount.name());
						break;
					case dialogueCountInShot:
						selections.put(Features.dialogueCountInShot.name(),counters.get("dialogueCountInShot"));
						break;
					case uniqueDialogueCountInShot:
						selections.put(Features.uniqueDialogueCountInShot.name(), 
								counters.get("uniqueDialogueCountInShot"));
						break;
					case actionCountInShot:
						selections.put(Features.actionCountInShot.name(), counters.get("actionCountInShot"));
						break;
					case lineCountInShot:
						selections.put(Features.lineCountInShot.name(), counters.get("lineCountInShot"));
						break;
					case sceneObjectType:
						selections.put(Features.sceneObjectType.name(), getObjectTypeOnLine(line.getLineNum(), data));
						break;
					case linesSinceObjectChange:
						selections.put(Features.linesSinceObjectChange.name(), counters.get("linesSinceObjectChange"));
						break;
					case linesSinceNewScene:
						selections.put(Features.linesSinceNewScene.name(), counters.get("linesSinceNewScene"));
						break;
					case intExt:
						selections.put(Features.intExt.name(), counters.get("ie"));
						break;
					case scriptObjectsInScene:
						selections.put(Features.scriptObjectsInScene.name(), counters.get("objCount"));
						if (counters.get("objCount") == 0) 
							skipFeatures.add(Features.scriptObjectsInScene.name());
						break;
					case dialogueBlocksInScene:
						selections.put(f.name(), counters.get("dialogueCount"));
						break;
					case actionBlocksInScene:
						selections.put(f.name(), counters.get("actionCount"));
						if (counters.get("actionCount") == 0) 
							skipFeatures.add(Features.actionBlocksInScene.name());
						break;
					case sceneLength:
						selections.put(f.name(), counters.get("sceneLength"));
						break;
					default:
						;
					}
				}
				switch (tF) {
				case Cut:
					//					skipFeatures.add(Features.cut.name());
					//					skipFeatures.add(Features.shotType.name());
					//					skipFeatures.add(Features.cleanType.name());
					//					skipFeatures.add(Features.motions.name());
					int cutOption = cutVector.getOption(selections, settings, skipFeatures);
					line.setNewShot(cutOption == 1); // || selections[0] == 1
					break;
				case ShotType:
					//					skipFeatures.add(Features.shotType.name());
					//					skipFeatures.add(Features.cleanType.name());
					//					skipFeatures.add(Features.motions.name());
					int shotOption = shotVector.getOption(selections, settings, skipFeatures);
					line.setShotType(shotOption);
					break;
				case CleanType:
					//					skipFeatures.add(Features.cleanType.name());
					//					skipFeatures.add(Features.motions.name());
					int cleanOption = cleanVector.getOption(selections, settings, skipFeatures);
					line.setCleanType(cleanOption);
					break;
				case Motion:
					//					skipFeatures.add(Features.motions.name());
					int motionOption = motionVector.getOption(selections, settings, skipFeatures);
					line.setMotion(motionOption);
					break;
				}

				updateCounters(data, counters, line);
			}
		}
		return data;
	}

	private static void updateCounters(AllData data,
			HashMap<String, Integer> counters, LineData line) {
		int linesSinceCut = counters.get("linesSinceCut");
		linesSinceCut = (line.isNewShot()) ? 0 : linesSinceCut + 1;
		if (linesSinceCut >= Constants.MAXLINESSINCECUT)
			linesSinceCut = Constants.MAXLINESSINCECUT - 1;
		counters.put("linesSinceCut", linesSinceCut);

		int oldShotType = counters.get("oldShotType");
		int linesSinceShotTypeChange = counters.get("linesSinceShotTypeChange");
		if (line.getShotType() == oldShotType)
			linesSinceShotTypeChange += 1;
		else {
			linesSinceShotTypeChange = 0;
			oldShotType = line.getShotType();
		}
		if (linesSinceShotTypeChange >= Constants.MAXLINESSINCESHOTTYPECHANGE)
			linesSinceShotTypeChange = Constants.MAXLINESSINCESHOTTYPECHANGE - 1;
		counters.put("linesSinceShotTypeChange", linesSinceShotTypeChange);
		counters.put("oldShotType", oldShotType);
		
		//linesSinceCleanChange
		int oldCleanType = counters.get("oldCleanType");
		int linesSinceCleanChange = counters.get("linesSinceCleanChange");
		if (line.getCleanType() == oldCleanType)
			linesSinceCleanChange += 1;
		else {
			linesSinceCleanChange = 0;
			oldCleanType = line.getCleanType();
		}
		if (linesSinceCleanChange >= Constants.MAXLINESSINCECLEANCHANGE)
			linesSinceCleanChange = Constants.MAXLINESSINCECLEANCHANGE - 1;
		counters.put("linesSinceCleanChange", linesSinceCleanChange);
		counters.put("oldCleanType", oldCleanType);

		//tilt count
		int oldMotion = counters.get("oldMotion");
		int tiltCount = counters.get("tiltCount");
		if (oldMotion == Motion.Tilt.ordinal())
			tiltCount++;
		else
			tiltCount=0;
		if (tiltCount == Constants.MAXMOTIONCOUNT)
			tiltCount = Constants.MAXMOTIONCOUNT - 1;
		counters.put("tiltCount", tiltCount);

		int zoomCount = counters.get("zoomCount");
		if (oldMotion == Motion.Zoom.ordinal())
			zoomCount++;
		else
			zoomCount=0;
		if (zoomCount == Constants.MAXMOTIONCOUNT)
			zoomCount = Constants.MAXMOTIONCOUNT - 1;
		counters.put("zoomCount", zoomCount);

		int panCount = counters.get("panCount");
		if (oldMotion == Motion.Pan.ordinal())
			panCount++;
		else
			panCount=0;
		if (panCount == Constants.MAXMOTIONCOUNT)
			panCount = Constants.MAXMOTIONCOUNT - 1;
		counters.put("panCount", panCount);

		//linesSinceMotionChange
		int linesSinceMotionChange = counters.get("linesSinceMotionChange");
		if (line.getMotion() == oldMotion)
			linesSinceMotionChange += 1;
		else {
			linesSinceMotionChange = 0;
			oldMotion = line.getMotion();
		}
		if (linesSinceMotionChange >= Constants.MAXLINESSINCEMOTIONCHANGE)
			linesSinceMotionChange = Constants.MAXLINESSINCEMOTIONCHANGE - 1;
		counters.put("linesSinceMotionChange", linesSinceMotionChange);
		counters.put("oldMotion", oldMotion);

		//linesSinceObjectChange
		int linesSinceObjectChange = counters.get("linesSinceObjectChange");
		if (line.getLineNum() < data.getDataLines().size() - 1) {
			linesSinceObjectChange =
					(getObjectTypeOnLine(line.getLineNum(), data) == getObjectTypeOnLine(line.getLineNum() + 1, data))
					? 0 :linesSinceObjectChange + 1;
			if (linesSinceObjectChange >= Constants.MAXLINESSINCEOBJECTCHANGE)
				linesSinceObjectChange = Constants.MAXLINESSINCEOBJECTCHANGE - 1;
		}
		counters.put("linesSinceObjectChange", linesSinceObjectChange);

		//lines since new scene
		int linesSinceNewScene = counters.get("linesSinceNewScene");
		linesSinceNewScene = (line.getSceneNum()!= null) ? 0 : linesSinceNewScene + 1;
		if (linesSinceNewScene >= Constants.MAXLINESSINCENEWSCENE)
			linesSinceNewScene = Constants.MAXLINESSINCENEWSCENE - 1;
		counters.put("linesSinceNewScene", linesSinceNewScene);
	}

	private static void updateNewSceneCounters(AllData data,
			HashMap<String, Integer> counters, LineData line) {
		int sceneNum = line.getSceneNum();
		counters.put("sceneNum", sceneNum);
		//				data.getScript().getSceneObjects().get(data.getScript().getLines()[line.getLineNum()]);
		counters.put("ie", 
				Util.getIntExt(data.getScript().getScenes().get(line.getSceneNum()-1).getWhere()).ordinal());
		int objCount = data.getScript().getScenes().get(sceneNum-1).getObjects().size();
		if (objCount >= Constants.MAXSCRIPTOBJECTSINSCENE)
			objCount = Constants.MAXSCRIPTOBJECTSINSCENE-1;
		counters.put("objCount", objCount);
		int dialogueCount = data.getScript().getSceneObjectCount(sceneNum-1, SceneObjectType.dialogue);
		if (dialogueCount >= Constants.MAXSCENEOBJECTS)
			dialogueCount = Constants.MAXSCENEOBJECTS-1;
		counters.put("dialogueCount", dialogueCount);
		int actionCount = data.getScript().getSceneObjectCount(sceneNum-1, SceneObjectType.action);
		if (actionCount >= Constants.MAXSCENEOBJECTS)
			actionCount = Constants.MAXSCENEOBJECTS-1;
		counters.put("actionCount", actionCount);
		int lineNum = line.getLineNum();
		int sceneLength = 0;
		do {
			sceneLength++;
			lineNum++;
		} while (lineNum < data.getDataLines().size()
				&& data.getDataLines().get(lineNum).getSceneNum() == null
				&& sceneLength < Constants.MAXSCENELENGTH - 1);
		counters.put("sceneLength", sceneLength);
	}

	private static HashMap<String, Integer> createCounters(AllData data) {
		HashMap<String, Integer> counters = new HashMap<String, Integer>();
		counters.put("oldShotType", data.getDataLines().get(0).getShotType());
		counters.put("oldCleanType", data.getDataLines().get(0).getCleanType());
		counters.put("oldMotion", data.getDataLines().get(0).getMotion());
		
		counters.put("linesSinceCut", 0);
		counters.put("linesSinceShotTypeChange", 0);
		counters.put("linesSinceCleanChange", 0);
		counters.put("linesSinceMotionChange", 0);
		counters.put("tiltCount", 0);
		counters.put("zoomCount", 0);
		counters.put("panCount", 0);
		counters.put("objCount", 0);
		counters.put("dialogueCount", 0);
		counters.put("actionCount", 0);
		counters.put("linesSinceObjectChange", 0);
		counters.put("linesSinceNewScene", 0);
		counters.put("ie", IntExt.none.ordinal());
		counters.put("sceneNum", 1);
		counters.put("sceneLength", 1);
		return counters;
	}
	
	/**
	 * Returns true of line is the first line of the script, a scene or a shot
	 * @param line The line being examines
	 * @return if skippable
	 */
	private static boolean skippable(LineData line) {
		return (line.getLineNum() == 0 	|| line.isNewShot() || line.getSceneNum() != null);
	}

	private static void updateShotCounts(AllData data, HashMap<String, Integer> counters, int line) {
		int charCount = 0;
		int uniqueCharCount = 0;
		int actionCount = 0;
		int lineCount = 0;
		int curSceneObject = 0;
		ArrayList<Integer> chars = new ArrayList<Integer>();
		do 
		{
			int nextSceneObject = data.getScript().getLines()[line + lineCount];
			if (nextSceneObject != curSceneObject) {
				curSceneObject = nextSceneObject;
				SceneObject obj= data.getScript().getSceneObjects().get(curSceneObject);
				if (obj.getType().equals(SceneObjectType.dialogue.name())) {
					charCount++;
					Integer character = ((Dialogue)obj).getCharacter();
					if (!chars.contains(character)) {
						chars.add(character);
						uniqueCharCount ++;
					}
				}
				if (obj.getType().equals(SceneObjectType.dialogue.name())) {
					actionCount++;
				}
				
			}
			lineCount++;
		} while ((line + lineCount) < data.getDataLines().size()
				&& !data.getDataLines().get(line + lineCount).isNewShot());
		if (charCount >= Constants.MAXSCENEOBJECTS)
			charCount = Constants.MAXSCENEOBJECTS - 1;
		if (uniqueCharCount >= Constants.MAXSCENEOBJECTS)
			uniqueCharCount = Constants.MAXSCENEOBJECTS - 1;
		if (actionCount >= Constants.MAXSCENEOBJECTS)
			actionCount = Constants.MAXSCENEOBJECTS - 1;
		if (lineCount >= Constants.MAXLINECOUNTINSHOT)
			lineCount = Constants.MAXLINECOUNTINSHOT - 1;
		counters.put("dialogueCountInShot", charCount);
		counters.put("uniqueDialogueCountInShot", uniqueCharCount);
		counters.put("actionCountInShot", actionCount);
		counters.put("lineCountInShot", lineCount);
//		if (line == 53) {
//			System.out.println("dialogueCountInShot " + charCount);
//			System.out.println("unique " + uniqueCharCount);
//			System.out.println("action " + actionCount);
//			System.out.println("line " + lineCount);
//		}
	}
}
